@@ -68,7 +68,8 @@ from __future__ import division
 #__all__ = ['mat']
 
 import numpy as np
-from shenfun.optimization.cython import Matvec
+import sympy as sp
+from shenfun.optimization import cython
 from shenfun.matrixbase import SpectralMatrix
 from shenfun.utilities import inheritdocstrings
 from shenfun.la import TDMA as neumann_TDMA
@@ -100,6 +101,9 @@ def get_ck(N, quad):
         ck[-1] = 2
     return ck
 
+def dmax(N, M, d):
+    Z = min(N, M)
+    return Z-abs(d)+min(max((M-N)*int(d/abs(d)), 0), abs(d))
 
 @inheritdocstrings
 class BDDmat(SpectralMatrix):
@@ -118,39 +122,44 @@ class BDDmat(SpectralMatrix):
     and :math:`\phi_j` is a Shen Dirichlet basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SD)
         ck = get_ck(test[0].N, test[0].quad)
         d = {0: np.pi/2*(ck[:-2]+ck[2:]),
              2: np.array([-np.pi/2])}
         d[-2] = d[2]
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self.solve = TDMA(self)
 
     def matvec(self, v, c, format='cython', axis=0):
         N, M = self.shape
         c.fill(0)
+        # Cython implementation only handles square matrix
+        if not M == N:
+            format = 'csr'
+
         if format == 'cython' and v.ndim == 3:
             ld = self[-2]*np.ones(M-2)
-            Matvec.Tridiagonal_matvec3D_ptr(v, c, ld, self[0], ld, axis)
+            cython.Matvec.Tridiagonal_matvec3D_ptr(v, c, ld, self[0], ld, axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 2:
             ld = self[-2]*np.ones(M-2)
-            Matvec.Tridiagonal_matvec2D_ptr(v, c, ld, self[0], ld, axis)
+            cython.Matvec.Tridiagonal_matvec2D_ptr(v, c, ld, self[0], ld, axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 1:
             ld = self[-2]*np.ones(M-2)
-            Matvec.Tridiagonal_matvec(v, c, ld, self[0], ld)
+            cython.Matvec.Tridiagonal_matvec(v, c, ld, self[0], ld)
             self.scale_array(c)
         elif format == 'self':
             if axis > 0:
                 v = np.moveaxis(v, axis, 0)
                 c = np.moveaxis(c, axis, 0)
-            s = (slice(None),)+(np.newaxis,)*(v.ndim-1) # broadcasting
-            c[:(N-2)] = self[2]*v[2:N]
-            c[:N] += self[0][s]*v[:N]
-            c[2:N] += self[-2]*v[:(N-2)]
+            s = (slice(0, M),)+(np.newaxis,)*(v.ndim-1) # broadcasting
+            sm2 = (slice(0, M-2),)+(np.newaxis,)*(v.ndim-1) # broadcasting
+            c[:(M-2)] = self[2][sm2]*v[2:M]
+            c[:M] += self[0][s]*v[:M]
+            c[2:M] += self[-2][sm2]*v[:(M-2)]
             if axis > 0:
                 v = np.moveaxis(v, 0, axis)
                 c = np.moveaxis(c, 0, axis)
@@ -182,16 +191,19 @@ class BNDmat(SpectralMatrix):
     For simplicity, the matrix is stored including the zero index row
     (:math:`k=0`)
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SD)
         N = test[0].N
-        ck = get_ck(N, test[0].quad)
-        k = np.arange(N-2, dtype=np.float)
+        M = trial[0].N
+        Q = min(N, M)
+        ck = get_ck(Q, test[0].quad)
+        k = np.arange(Q-2, dtype=np.float)
         d = {-2: -np.pi/2,
-              0: np.pi/2.*(ck[:-2]+ck[2:]*(k/(k+2))**2),
-              2: -np.pi/2*(k[:N-4]/(k[:N-4]+2))**2}
-        SpectralMatrix.__init__(self, d, test, trial)
+              0: np.pi/2.*(ck[:-2]+ck[2:]*(k/(k+2))**2)}
+        d2 = -np.pi/2*(k/(k+2))**2
+        d[2] = d2[:dmax(N-2, M-2, 2)]
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='csr', axis=0):
         c = super(BNDmat, self).matvec(v, c, format=format, axis=axis)
@@ -221,27 +233,32 @@ class BDNmat(SpectralMatrix):
     For simplicity, the matrix is stored including the zero index column
     (:math:`j=0`)
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SN)
         N = test[0].N
-        ck = get_ck(N, test[0].quad)
-        k = np.arange(N-2, dtype=np.float)
-        d = {-2: -np.pi/2*(k[:N-4]/(k[:N-4]+2))**2,
-              0:  np.pi/2.*(ck[:-2]+ck[2:]*(k/(k+2))**2),
-              2: -np.pi/2}
-        SpectralMatrix.__init__(self, d, test, trial)
+        M = trial[0].N
+        Q = min(N, M)
+        ck = get_ck(Q, test[0].quad)
+        k = np.arange(Q-2, dtype=np.float)
+        d = {0:  np.pi/2.*(ck[:-2]+ck[2:]*(k/(k+2))**2),
+             2: -np.pi/2}
+        d[-2] = (-np.pi/2*(k/(k+2))**2)[:dmax(N-2, M-2, -2)]
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='cython', axis=0):
         c.fill(0)
+        N, M = self.shape
+        if not M == N:
+            format = 'csr'
         if format == 'cython' and v.ndim == 3:
-            Matvec.BDN_matvec3D_ptr(v, c, self[-2], self[0], self[2], axis)
+            cython.Matvec.BDN_matvec3D_ptr(v, c, self[-2], self[0], self[2], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 2:
-            Matvec.BDN_matvec2D_ptr(v, c, self[-2], self[0], self[2], axis)
+            cython.Matvec.BDN_matvec2D_ptr(v, c, self[-2], self[0], self[2], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 1:
-            Matvec.BDN_matvec1D_ptr(v, c, self[-2], self[0], self[2])
+            cython.Matvec.BDN_matvec1D_ptr(v, c, self[-2], self[0], self[2])
             self.scale_array(c)
         else:
             c = super(BDNmat, self).matvec(v, c, format=format, axis=axis)
@@ -270,7 +287,7 @@ class BNTmat(SpectralMatrix):
     (:math:`k=0`)
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], CB)
         SpectralMatrix.__init__(self, {}, test, trial)
@@ -304,7 +321,7 @@ class BNBmat(SpectralMatrix):
     (:math:`k=0`)
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SB)
         SpectralMatrix.__init__(self, {}, test, trial)
@@ -333,24 +350,43 @@ class BTTmat(SpectralMatrix):
 
     and :math:`T_j` is the jth order Chebyshev function of the first kind.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], CB)
         assert isinstance(trial[0], CB)
-        ck = get_ck(test[0].N, test[0].quad)
+        ck = get_ck(min(test[0].N, trial[0].N), test[0].quad)
         SpectralMatrix.__init__(self, {0: np.pi/2*ck}, test, trial)
 
-    def matvec(self, v, c, format='self', axis=0):
+    def matvec(self, v, c, format='csr', axis=0):
         c.fill(0)
+        N, M = self.shape
+        if not M == N:
+            format = 'csr'
         if format == 'self':
             s = [np.newaxis,]*v.ndim # broadcasting
-            s[axis] = slice(None)
-            c[:] = self[0][tuple(s)]*v
+            d = [slice(0, m) for m in v.shape]
+            N, M = self.shape
+            s[axis] = slice(0, M)
+            s = tuple(s)
+            c[d] = self[0][s]*v[d]
             self.scale_array(c)
         else:
             c = super(BTTmat, self).matvec(v, c, format=format, axis=axis)
 
         return c
 
+    def solve(self, b, u=None, axis=0):
+        s = self.trialfunction[0].slice()
+        if u is None:
+            u = b
+        else:
+            assert u.shape == b.shape
+        sl = [np.newaxis]*u.ndim
+        sl[axis] = s
+        sl = tuple(sl)
+        ss = self.trialfunction[0].sl[s]
+        d = (1./self.scale)/self[0]
+        u[ss] = b[ss]*d[sl]
+        return u
 
 @inheritdocstrings
 class BNNmat(SpectralMatrix):
@@ -371,16 +407,21 @@ class BNNmat(SpectralMatrix):
     The matrix is stored including the zero index row and column
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SN)
         N = test[0].N
-        ck = get_ck(N, test[0].quad)
-        k = np.arange(N-2, dtype=np.float)
-        d = {0: np.pi/2*(ck[:-2]+ck[2:]*(k[:]/(k[:]+2))**4),
-             2: -np.pi/2*((k[2:]-2)/(k[2:]))**2}
-        d[-2] = d[2]
-        SpectralMatrix.__init__(self, d, test, trial)
+        M = trial[0].N
+        Q = min(N, M)
+        ck = get_ck(Q, test[0].quad)
+        k = np.arange(Q-2, dtype=np.float)
+        d = {0: np.pi/2*(ck[:-2]+ck[2:]*(k[:]/(k[:]+2))**4)}
+
+        dp = dmax(N-2, M-2, 2)
+        d[2] = -np.pi/2*(k[:dp]/(k[:dp]+2))**2
+        dm = dmax(N-2, M-2, -2)
+        d[-2] = -np.pi/2*(k[:dm]/(k[:dm]+2))**2
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self.solve = neumann_TDMA(self)
 
     def matvec(self, v, c, format='csr', axis=0):
@@ -408,14 +449,16 @@ class BDTmat(SpectralMatrix):
     :math:`\phi_k` is the Shen Dirichlet basis function and :math:`T_j` is the
     Chebyshev basis function.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], CB)
-        N = test[0].N
-        ck = get_ck(N, test[0].quad)
-        d = {0: np.pi/2*ck[:N-2],
-             2: -np.pi/2*ck[2:]}
-        SpectralMatrix.__init__(self, d, test, trial)
+        N = test[0].N-2
+        M = trial[0].N
+        Q = min(N, M)
+        ck = get_ck(Q+2, test[0].quad)
+        d = {0: np.pi/2*ck[:Q],
+             2: -np.pi/2*ck[2:(dmax(N, M, 2)+2)]}
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
 @inheritdocstrings
@@ -435,14 +478,16 @@ class BTDmat(SpectralMatrix):
     :math:`\phi_j` is the Shen Dirichlet basis function and :math:`T_k` is the
     Chebyshev basis function.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], CB)
         assert isinstance(trial[0], SD)
         N = test[0].N
+        M = trial[0].N-2
+        Q = min(N, M)
         ck = get_ck(N, test[0].quad)
-        d = {-2: -np.pi/2*ck[2:],
-              0: np.pi/2*ck[:N-2]}
-        SpectralMatrix.__init__(self, d, test, trial)
+        d = {0: np.pi/2*ck[:Q]}
+        d[-2] = -np.pi/2*ck[2:(dmax(N, M, -2)+2)]
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
 @inheritdocstrings
@@ -462,7 +507,7 @@ class BTNmat(SpectralMatrix):
     :math:`\phi_j` is the Shen Neumann basis function and :math:`T_k` is the
     Chebyshev basis function.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], CB)
         assert isinstance(trial[0], SN)
         N = test[0].N
@@ -470,7 +515,7 @@ class BTNmat(SpectralMatrix):
         k = np.arange(N, dtype=np.float)
         d = {-2: -np.pi/2*ck[2:]*((k[2:]-2)/k[2:])**2,
               0: np.pi/2*ck[:-2]}
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
 @inheritdocstrings
@@ -489,24 +534,31 @@ class BBBmat(SpectralMatrix):
 
     and :math:`\psi_j` is the Shen Biharmonic basis function.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         from shenfun.la import PDMA
         N = test[0].N
-        ck = get_ck(N, test[0].quad)
-        k = np.arange(N-4, dtype=np.float)
-        d = {4: (k[:-4]+1)/(k[:-4]+3)*np.pi/2,
-             2: -((k[:-2]+2)/(k[:-2]+3) + (k[:-2]+4)*(k[:-2]+1)/((k[:-2]+5)*(k[:-2]+3)))*np.pi,
-             0: (ck[:N-4] + 4*((k+2)/(k+3))**2 + ck[4:]*((k+1)/(k+3))**2)*np.pi/2.}
-        d[-2] = d[2]
-        d[-4] = d[4]
-        SpectralMatrix.__init__(self, d, test, trial)
+        M = trial[0].N
+        Q = min(N, M)
+        ck = get_ck(Q, test[0].quad)
+        k = np.arange(Q-4, dtype=np.float)
+        d = {0: (ck[:Q-4] + 4*((k+2)/(k+3))**2 + ck[4:]*((k+1)/(k+3))**2)*np.pi/2.}
+        d4 = (k+1)/(k+3)*np.pi/2
+        d2 = -((k+2)/(k+3)+(k+4)*(k+1)/((k+5)*(k+3)))*np.pi
+        d[2] = d2[:dmax(N-4, M-4, 2)]
+        d[4] = d4[:dmax(N-4, M-4, 4)]
+        d[-2] = d2[:dmax(N-4, M-4, -2)]
+        d[-4] = d4[:dmax(N-4, M-4, -4)]
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self.solve = PDMA(self)
 
     def matvec(self, v, c, format='cython', axis=0):
         c.fill(0)
-        N = self.shape[0]
+        N, M = self.shape
+        if not M == N:
+            format = 'csr'
+
         if format == 'self':
             if axis > 0:
                 v = np.moveaxis(v, axis, 0)
@@ -525,17 +577,17 @@ class BBBmat(SpectralMatrix):
             self.scale_array(c)
 
         elif format == 'cython' and v.ndim == 3:
-            Matvec.Pentadiagonal_matvec3D_ptr(v, c, self[-4], self[-2], self[0],
+            cython.Matvec.Pentadiagonal_matvec3D_ptr(v, c, self[-4], self[-2], self[0],
                                               self[2], self[4], axis)
             self.scale_array(c)
 
         elif format == 'cython' and v.ndim == 2:
-            Matvec.Pentadiagonal_matvec2D_ptr(v, c, self[-4], self[-2], self[0],
+            cython.Matvec.Pentadiagonal_matvec2D_ptr(v, c, self[-4], self[-2], self[0],
                                               self[2], self[4], axis)
             self.scale_array(c)
 
         elif format == 'cython' and v.ndim == 1:
-            Matvec.Pentadiagonal_matvec(v, c, self[-4], self[-2], self[0],
+            cython.Matvec.Pentadiagonal_matvec(v, c, self[-4], self[-2], self[0],
                                         self[2], self[4])
             self.scale_array(c)
         else:
@@ -561,7 +613,7 @@ class BBDmat(SpectralMatrix):
     and :math:`\phi_j` is the Shen Dirichlet basis function and :math:`\psi_k`
     the Shen Biharmonic basis function.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -573,7 +625,7 @@ class BBDmat(SpectralMatrix):
               0: (ck[:N-4] + a)*np.pi/2,
               2: -(a+b*ck[4:])*np.pi/2,
               4: b[:-2]*np.pi/2}
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='cython', axis=0):
         c.fill(0)
@@ -594,13 +646,13 @@ class BBDmat(SpectralMatrix):
             self.scale_array(c)
 
         elif format == 'cython' and v.ndim == 3:
-            Matvec.BBD_matvec3D_ptr(v, c, self[-2], self[0], self[2], self[4], axis)
+            cython.Matvec.BBD_matvec3D_ptr(v, c, self[-2], self[0], self[2], self[4], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 2:
-            Matvec.BBD_matvec2D_ptr(v, c, self[-2], self[0], self[2], self[4], axis)
+            cython.Matvec.BBD_matvec2D_ptr(v, c, self[-2], self[0], self[2], self[4], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 1:
-            Matvec.BBD_matvec1D_ptr(v, c, self[-2], self[0], self[2], self[4])
+            cython.Matvec.BBD_matvec1D_ptr(v, c, self[-2], self[0], self[2], self[4])
             self.scale_array(c)
         else:
             c = super(BBDmat, self).matvec(v, c, format=format, axis=axis)
@@ -627,24 +679,24 @@ class CDNmat(SpectralMatrix):
     For simplicity, the matrix is stored including the zero index row (k=0)
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SN)
         N = test[0].N
         k = np.arange(N-2, dtype=np.float)
         d = {-1: -((k[1:]-1)/(k[1:]+1))**2*(k[1:]+1)*np.pi,
               1: (k[:-1]+1)*np.pi}
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='cython', axis=0):
         if format == 'cython' and v.ndim == 3:
-            Matvec.CDN_matvec3D_ptr(v, c, self[-1], self[1], axis)
+            cython.Matvec.CDN_matvec3D_ptr(v, c, self[-1], self[1], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 2:
-            Matvec.CDN_matvec2D_ptr(v, c, self[-1], self[1], axis)
+            cython.Matvec.CDN_matvec2D_ptr(v, c, self[-1], self[1], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 1:
-            Matvec.CDN_matvec1D_ptr(v, c, self[-1], self[1])
+            cython.Matvec.CDN_matvec1D_ptr(v, c, self[-1], self[1])
             self.scale_array(c)
         else:
             c = super(CDNmat, self).matvec(v, c, format=format, axis=axis)
@@ -668,14 +720,14 @@ class CDDmat(SpectralMatrix):
 
     and :math:`\phi_k` is the Shen Dirichlet basis function.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SD)
         N = test[0].N
         k = np.arange(N, dtype=np.float)
         d = {-1: -(k[1:N-2]+1)*np.pi,
               1: (k[:(N-3)]+1)*np.pi}
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='cython', axis=0):
         N = self.shape[0]
@@ -694,13 +746,13 @@ class CDDmat(SpectralMatrix):
             self.scale_array(c)
 
         elif format == 'cython' and v.ndim == 3:
-            Matvec.CDD_matvec3D_ptr(v, c, self[-1], self[1], axis)
+            cython.Matvec.CDD_matvec3D_ptr(v, c, self[-1], self[1], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 2:
-            Matvec.CDD_matvec2D_ptr(v, c, self[-1], self[1], axis)
+            cython.Matvec.CDD_matvec2D_ptr(v, c, self[-1], self[1], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 1:
-            Matvec.CDD_matvec1D_ptr(v, c, self[-1], self[1])
+            cython.Matvec.CDD_matvec1D_ptr(v, c, self[-1], self[1])
             self.scale_array(c)
         else:
             c = super(CDDmat, self).matvec(v, c, format=format, axis=axis)
@@ -728,7 +780,7 @@ class CNDmat(SpectralMatrix):
     For simplicity, the matrix is stored including the zero index coloumn (j=0)
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -737,7 +789,7 @@ class CNDmat(SpectralMatrix):
               1: -(2-k[:-1]**2/(k[:-1]+2)**2*(k[:-1]+3))*np.pi}
         for i in range(3, N-1, 2):
             d[i] = -(1-k[:-i]**2/(k[:-i]+2)**2)*2*np.pi
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='csr', axis=0):
         c = super(CNDmat, self).matvec(v, c, format=format, axis=axis)
@@ -764,7 +816,7 @@ class CTDmat(SpectralMatrix):
     :math:`\phi_j` is the Shen Dirichlet basis function and :math:`T_k` is the
     Chebyshev basis function.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], CB)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -773,7 +825,7 @@ class CTDmat(SpectralMatrix):
               1: -2*np.pi}
         for i in range(3, N-2, 2):
             d[i] = -2*np.pi
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
 class CDTmat(SpectralMatrix):
@@ -792,13 +844,13 @@ class CDTmat(SpectralMatrix):
     :math:`\phi_k` is the Shen Dirichlet basis function and :math:`T_j` is the
     Chebyshev basis function.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], CB)
         N = test[0].N
         k = np.arange(N, dtype=np.float)
         d = {1: np.pi*(k[:N-2]+1)}
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
 @inheritdocstrings
@@ -818,7 +870,7 @@ class CBDmat(SpectralMatrix):
     :math:`\phi_j` is the Shen Dirichlet basis and :math:`\psi_k` the Shen
     Biharmonic basis function.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -826,7 +878,7 @@ class CBDmat(SpectralMatrix):
         d = {-1: -(k[1:N-4]+1)*np.pi,
               1: 2*(k[:N-4]+1)*np.pi,
               3: -(k[:N-5]+1)*np.pi}
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='cython', axis=0):
         N, M = self.shape
@@ -844,13 +896,13 @@ class CBDmat(SpectralMatrix):
                 v = np.moveaxis(v, 0, axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 3:
-            Matvec.CBD_matvec3D_ptr(v, c, self[-1], self[1], self[3], axis)
+            cython.Matvec.CBD_matvec3D_ptr(v, c, self[-1], self[1], self[3], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 2:
-            Matvec.CBD_matvec2D_ptr(v, c, self[-1], self[1], self[3], axis)
+            cython.Matvec.CBD_matvec2D_ptr(v, c, self[-1], self[1], self[3], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 1:
-            Matvec.CBD_matvec(v, c, self[-1], self[1], self[3])
+            cython.Matvec.CBD_matvec(v, c, self[-1], self[1], self[3])
             self.scale_array(c)
         else:
             c = super(CBDmat, self).matvec(v, c, format=format, axis=axis)
@@ -872,7 +924,7 @@ class CDBmat(SpectralMatrix):
     :math:`\phi_k` is the Shen Dirichlet basis function and :math:`\psi_j` the
     Shen Biharmonic basis function.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SB)
         N = test[0].N
@@ -880,7 +932,7 @@ class CDBmat(SpectralMatrix):
         d = {-3: (k[3:-2]-2)*(k[3:-2]+1)/k[3:-2]*np.pi,
              -1: -2*(k[1:-3]+1)**2/(k[1:-3]+2)*np.pi,
               1: (k[:-5]+1)*np.pi}
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='cython', axis=0):
         N, M = self.shape
@@ -900,13 +952,13 @@ class CDBmat(SpectralMatrix):
             self.scale_array(c)
 
         elif format == 'cython' and v.ndim == 3:
-            Matvec.CDB_matvec3D_ptr(v, c, self[-3], self[-1], self[1], axis)
+            cython.Matvec.CDB_matvec3D_ptr(v, c, self[-3], self[-1], self[1], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 2:
-            Matvec.CDB_matvec2D_ptr(v, c, self[-3], self[-1], self[1], axis)
+            cython.Matvec.CDB_matvec2D_ptr(v, c, self[-3], self[-1], self[1], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 1:
-            Matvec.CDB_matvec(v, c, self[-3], self[-1], self[1])
+            cython.Matvec.CDB_matvec(v, c, self[-3], self[-1], self[1])
             self.scale_array(c)
         else:
             c = super(CDBmat, self).matvec(v, c, format=format, axis=axis)
@@ -931,7 +983,7 @@ class ABBmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Biharmonic basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
@@ -939,7 +991,7 @@ class ABBmat(SpectralMatrix):
         d = {-2: 2*(k[2:]-1)*(k[2:]+2)*np.pi,
               0: -4*((k+1)*(k+2)**2)/(k+3)*np.pi,
               2: 2*(k[:-2]+1)*(k[:-2]+2)*np.pi}
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='cython', axis=0):
         N = self.shape[0]
@@ -959,13 +1011,13 @@ class ABBmat(SpectralMatrix):
             self.scale_array(c)
 
         elif format == 'cython' and v.ndim == 3:
-            Matvec.Tridiagonal_matvec3D_ptr(v, c, self[-2], self[0], self[2], axis)
+            cython.Matvec.Tridiagonal_matvec3D_ptr(v, c, self[-2], self[0], self[2], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 2:
-            Matvec.Tridiagonal_matvec2D_ptr(v, c, self[-2], self[0], self[2], axis)
+            cython.Matvec.Tridiagonal_matvec2D_ptr(v, c, self[-2], self[0], self[2], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 1:
-            Matvec.Tridiagonal_matvec(v, c, self[-2], self[0], self[2])
+            cython.Matvec.Tridiagonal_matvec(v, c, self[-2], self[0], self[2])
             self.scale_array(c)
 
         else:
@@ -991,7 +1043,7 @@ class ADDmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Dirichlet basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -999,23 +1051,23 @@ class ADDmat(SpectralMatrix):
         d = {0: -2*np.pi*(k[:N-2]+1)*(k[:N-2]+2)}
         for i in range(2, N-2, 2):
             d[i] = -4*np.pi*(k[:-(i+2)]+1)
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
         # Following storage more efficient, but requires effort in iadd/isub...
-        #d = {0: -2*np.pi*(k[:N-2]+1)*(k[:N-2]+2),
-             #2: -4*np.pi*(k[:-4]+1)}
-        #for i in range(4, N-2, 2):
-            #d[i] = d[2][:2-i]
+#        d = {0: -2*np.pi*(k[:N-2]+1)*(k[:N-2]+2),
+#             2: -4*np.pi*(k[:-4]+1)}
+#        for i in range(4, N-2, 2):
+#            d[i] = d[2][:2-i]
     def matvec(self, v, c, format='cython', axis=0):
         c.fill(0)
         if format == 'cython' and v.ndim == 3:
-            Matvec.ADD_matvec3D_ptr(v, c, self[0], axis)
+            cython.Matvec.ADD_matvec3D_ptr(v, c, self[0], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 2:
-            Matvec.ADD_matvec2D_ptr(v, c, self[0], axis)
+            cython.Matvec.ADD_matvec2D_ptr(v, c, self[0], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 1:
-            Matvec.ADD_matvec(v, c, self[0])
+            cython.Matvec.ADD_matvec(v, c, self[0])
             self.scale_array(c)
         else:
             c = super(ADDmat, self).matvec(v, c, format=format, axis=axis)
@@ -1035,7 +1087,7 @@ class ADDmat(SpectralMatrix):
         # Move axis to first
         if axis > 0:
             u = np.moveaxis(u, axis, 0)
-            if not u is b:
+            if u is not b:
                 b = np.moveaxis(b, axis, 0)
 
         bs = b[s]
@@ -1061,12 +1113,12 @@ class ADDmat(SpectralMatrix):
                 us[k] = bs[k] - d1[k]*so
             us[k] /= d[k]
 
-        self.testfunction[0].bc.apply_after(u, True)
+        u /= self.scale
+        self.testfunction[0].bc.set_boundary_dofs(u, True)
         if axis > 0:
             u = np.moveaxis(u, 0, axis)
-            if not u is b:
+            if u is not b:
                 b = np.moveaxis(b, 0, axis)
-        u /= self.scale
         return u
 
 
@@ -1087,7 +1139,7 @@ class ANNmat(SpectralMatrix):
     and :math:`\phi_k` is the Shen Neumann basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SN)
         N = test[0].N
@@ -1095,7 +1147,7 @@ class ANNmat(SpectralMatrix):
         d = {0: -2*np.pi*k**2*(k+1)/(k+2)}
         for i in range(2, N-2, 2):
             d[i] = -4*np.pi*(k[:-i]+i)**2*(k[:-i]+1)/(k[:-i]+2)**2
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='csr', axis=0):
         c = super(ANNmat, self).matvec(v, c, format=format, axis=axis)
@@ -1116,7 +1168,7 @@ class ANNmat(SpectralMatrix):
         # Move axis to first
         if axis > 0:
             u = np.moveaxis(u, axis, 0)
-            if not u is b:
+            if u is not b:
                 b = np.moveaxis(b, axis, 0)
 
         bs = b[s]
@@ -1152,7 +1204,7 @@ class ANNmat(SpectralMatrix):
 
         if axis > 0:
             u = np.moveaxis(u, 0, axis)
-            if not u is b:
+            if u is not b:
                 b = np.moveaxis(b, 0, axis)
         u /= self.scale
         return u
@@ -1175,7 +1227,7 @@ class ATTmat(SpectralMatrix):
     and :math:`\psi_k` is the Chebyshev basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], CB)
         assert isinstance(trial[0], CB)
         N = test[0].N
@@ -1183,7 +1235,7 @@ class ATTmat(SpectralMatrix):
         d = {}
         for j in range(2, N, 2):
             d[j] = k[j:]*(k[j:]**2-k[:-j]**2)*np.pi/2.
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
 @inheritdocstrings
@@ -1202,7 +1254,7 @@ class SBBmat(SpectralMatrix):
 
     and :math:`\psi_k` is the Shen Biharmonic basis function.
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
@@ -1213,18 +1265,18 @@ class SBBmat(SpectralMatrix):
         for j in range(2, N-4, 2):
             i = 8*(ki[:-j]+1)*(ki[:-j]+2)*(ki[:-j]*(ki[:-j]+4)+3*(ki[j:]+2)**2)
             d[j] = np.array(i*np.pi/(k[j:]+3))
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='cython', axis=0):
         c.fill(0)
         if format == 'cython' and v.ndim == 3:
-            Matvec.SBB_matvec3D_ptr(v, c, self[0], axis)
+            cython.Matvec.SBB_matvec3D_ptr(v, c, self[0], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 2:
-            Matvec.SBB_matvec2D_ptr(v, c, self[0], axis)
+            cython.Matvec.SBB_matvec2D_ptr(v, c, self[0], axis)
             self.scale_array(c)
         elif format == 'cython' and v.ndim == 1:
-            Matvec.SBBmat_matvec(v, c, self[0])
+            cython.Matvec.SBBmat_matvec(v, c, self[0])
             self.scale_array(c)
 
         else:
@@ -1235,8 +1287,8 @@ class SBBmat(SpectralMatrix):
 
 @inheritdocstrings
 class _Chebmatrix(SpectralMatrix):
-    def __init__(self, test, trial):
-        SpectralMatrix.__init__(self, {}, test, trial)
+    def __init__(self, test, trial, measure=1):
+        SpectralMatrix.__init__(self, {}, test, trial, measure=measure)
 
 
 class _ChebMatDict(dict):
