@@ -1,6 +1,7 @@
 """
 Module for defining bases in the Fourier family
 """
+import sympy as sp
 import numpy as np
 from mpi4py_fft import fftw
 from shenfun.spectralbase import SpectralBase, Transform, islicedict, slicedict
@@ -71,22 +72,21 @@ class FourierBase(SpectralBase):
             Number of quadrature points. Should be even for efficiency, but
             this is not required.
         padding_factor : float, optional
-                         Factor for padding backward transforms.
-                         padding_factor=1.5 corresponds to a 3/2-rule for
-                         dealiasing.
+            Factor for padding backward transforms. padding_factor=1.5
+            corresponds to a 3/2-rule for dealiasing.
         domain : 2-tuple of floats, optional
-                 The computational domain.
+            The computational domain.
         dealias_direct : bool, optional
-                         True for dealiasing using 2/3-rule. Must be used with
-                         padding_factor == 1.
+            True for dealiasing using 2/3-rule. Must be used with
+            padding_factor = 1.
     """
 
     def __init__(self, N, padding_factor=1., domain=(0, 2*np.pi),
-                 dealias_direct=False):
+                 dealias_direct=False, **kw):
         self.dealias_direct = dealias_direct
         self._k = None
         self._planned_axes = None  # Collapsing of axes means that this base can be used to plan transforms over several collapsed axes. Store the axes planned for here.
-        SpectralBase.__init__(self, N, '', padding_factor, domain)
+        SpectralBase.__init__(self, N, padding_factor=padding_factor, domain=domain)
 
     @staticmethod
     def family():
@@ -96,7 +96,7 @@ class FourierBase(SpectralBase):
     def boundary_condition():
         return 'Periodic'
 
-    def points_and_weights(self, N=None, map_true_domain=False):
+    def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
         if N is None:
             N = self.N
         points = np.arange(N, dtype=np.float)*2*np.pi/N
@@ -104,6 +104,12 @@ class FourierBase(SpectralBase):
             points = self.map_true_domain(points)
         return points, np.array([2*np.pi/N])
 
+    def sympy_basis(self, i=0, x=sp.symbols('x')):
+        return sp.exp(1j*i*x)
+
+    def sympy_weight(self, x=sp.symbols('x')):
+        return 2*sp.pi/self.N
+
     def evaluate_basis(self, x=None, i=0, output_array=None):
         if x is None:
             x = self.mesh(False, False)
@@ -136,8 +142,12 @@ class FourierBase(SpectralBase):
         return V
 
     # Reimplemented for efficiency (smaller array in *= when truncated)
-    #@profile
-    def forward(self, input_array=None, output_array=None, fast_transform=True):
+    def forward(self, input_array=None, output_array=None, fast_transform=True, padding_factor=1):
+        if padding_factor > 1:
+            assert self._padded_basis is not None
+            output_array = self._padded_basis.forward(input_array, output_array, fast_transform=fast_transform)
+            return output_array
+
         if fast_transform is False:
             return SpectralBase.forward(self, input_array, output_array, False)
 
@@ -192,10 +202,34 @@ class FourierBase(SpectralBase):
     def reference_domain(self):
         return (0., 2*np.pi)
 
+    @property
+    def is_orthogonal(self):
+        return True
+
+    def get_orthogonal(self):
+        return self
+
+    def mask_nyquist(self, u_hat, mask=None):
+        """Return array `u_hat` with zero Nyquist coefficients
+
+        Parameters
+        ----------
+        u_hat : array
+            Array to be masked
+        mask : array or None, optional
+            mask array, if not provided then get the mask by calling
+            :func:`get_mask_nyquist`
+        """
+        if mask is None:
+            mask = self.get_mask_nyquist(bcast=False)
+        if mask is not None:
+            u_hat *= mask
+        return u_hat
+
     def plan(self, shape, axis, dtype, options):
         if isinstance(axis, int):
             axis = [axis]
-        s = tuple(np.take(shape, axis))
+        s = list(np.take(shape, axis))
 
         if isinstance(self.forward, Transform):
             if self.forward.input_array.shape == shape and axis == self._planned_axes:
@@ -205,6 +239,9 @@ class FourierBase(SpectralBase):
         plan_fwd = self._xfftn_fwd
         plan_bck = self._xfftn_bck
 
+        self.axis = axis[-1]
+        self._planned_axes = axis
+
         if 'builders' in self._xfftn_fwd.__module__:
 
             opts = dict(
@@ -234,9 +271,9 @@ class FourierBase(SpectralBase):
                 threads=1,
             )
             opts.update(options)
+            threads = opts['threads']
             flags = (fftw.flag_dict[opts['planner_effort']],
                      fftw.flag_dict[opts['overwrite_input']])
-            threads = opts['threads']
 
             U = fftw.aligned(shape, dtype=dtype)
             xfftn_fwd = plan_fwd(U, s=s, axes=axis, threads=threads, flags=flags)
@@ -249,30 +286,46 @@ class FourierBase(SpectralBase):
             V.fill(0)
             U.fill(0)
             self._M = xfftn_fwd.get_normalization()
-        self.axis = axis[-1]
-        self._planned_axes = axis
 
         if self.padding_factor > 1.+1e-8:
             trunc_array = self._get_truncarray(shape, V.dtype)
             self.forward = Transform(self.forward, xfftn_fwd, U, V, trunc_array)
             self.backward = Transform(self.backward, xfftn_bck, trunc_array, V, U)
+            self.backward_uniform = self.backward
+
         else:
             self.forward = Transform(self.forward, xfftn_fwd, U, V, V)
             self.backward = Transform(self.backward, xfftn_bck, V, V, U)
+            self.backward_uniform = self.backward
 
         # scalar_product is not padded, just the forward/backward
         self.scalar_product = Transform(self.scalar_product, xfftn_fwd, U, V, V)
-        self.si = islicedict(axis=self.axis, dimensions=self.dimensions())
-        self.sl = slicedict(axis=self.axis, dimensions=self.dimensions())
+        self.si = islicedict(axis=self.axis, dimensions=self.dimensions)
+        self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
 
 
 class R2CBasis(FourierBase):
     """Fourier basis class for real to complex transforms
+
+    Parameters
+    ----------
+        N : int
+            Number of quadrature points. Should be even for efficiency, but
+            this is not required.
+        padding_factor : float, optional
+            Factor for padding backward transforms. padding_factor=1.5
+            corresponds to a 3/2-rule for dealiasing.
+        domain : 2-tuple of floats, optional
+            The computational domain.
+        dealias_direct : bool, optional
+            True for dealiasing using 2/3-rule. Must be used with
+            padding_factor = 1.
     """
 
     def __init__(self, N, padding_factor=1., domain=(0., 2.*np.pi),
-                 dealias_direct=False):
-        FourierBase.__init__(self, N, padding_factor, domain, dealias_direct)
+                 dealias_direct=False, **kw):
+        FourierBase.__init__(self, N, padding_factor=padding_factor,
+                             domain=domain, dealias_direct=dealias_direct)
         self.N = N
         #self._xfftn_fwd = pyfftw.builders.rfftn
         #self._xfftn_bck = pyfftw.builders.irfftn
@@ -280,7 +333,7 @@ class R2CBasis(FourierBase):
         self._xfftn_bck = fftw.irfftn
         self._sn = []
         self._sm = []
-        self.plan((int(np.floor(padding_factor*N)),), (0,), np.float, {})
+        self.plan((int(padding_factor*N),), (0,), np.float, {})
 
     def wavenumbers(self, bcast=True, scaled=False, eliminate_highest_freq=False):
         k = np.fft.rfftfreq(self.N, 1./self.N).astype(int)
@@ -292,6 +345,24 @@ class R2CBasis(FourierBase):
             k = self.broadcast_to_ndims(k)
         return k
 
+    def get_mask_nyquist(self, bcast=True):
+        """Return None or an array with zeros for Nyquist coefficients and one otherwise
+
+        Parameters
+        ----------
+        bcast : boolean, optional
+            If True then broadcast returned mask array to dimensions of the
+            :class:`TensorProductSpace` this base belongs to.
+        """
+        if self.N % 2 == 0:
+            f = np.ones(self.N//2+1, dtype=int)
+            f[-1] = 0
+        else:
+            return None
+        if bcast is True:
+            f = self.broadcast_to_ndims(f)
+        return f
+
     def _get_truncarray(self, shape, dtype):
         shape = list(shape)
         shape[self.axis] = int(shape[self.axis] / self.padding_factor)
@@ -301,12 +372,12 @@ class R2CBasis(FourierBase):
     def slice(self):
         return slice(0, self.N//2+1)
 
-    def allocated_shape(self, forward_output=True):
+    def shape(self, forward_output=True):
         if forward_output:
             return self.N//2+1
-        return self.N
+        return int(np.floor(self.padding_factor*self.N))
 
-    def vandermonde_evaluate_expansion_all(self, input_array, output_array):
+    def vandermonde_evaluate_expansion_all(self, input_array, output_array, x=None):
         assert abs(self.padding_factor-1) < 1e-8
         assert self.N == output_array.shape[self.axis]
         points = self.points_and_weights()[0]
@@ -371,7 +442,7 @@ class R2CBasis(FourierBase):
         return output_array
 
     def _truncation_forward(self, padded_array, trunc_array):
-        if self.padding_factor > 1.0+1e-8:
+        if not id(trunc_array) == id(padded_array):
             trunc_array.fill(0)
             N = trunc_array.shape[self.axis]
             s = self.sl[slice(0, N)]
@@ -382,10 +453,10 @@ class R2CBasis(FourierBase):
                 trunc_array[s1] *= 2
 
     def _padding_backward(self, trunc_array, padded_array):
-        if self.padding_factor > 1.0+1e-8:
+        if not id(trunc_array) == id(padded_array):
             padded_array.fill(0)
             N = trunc_array.shape[self.axis]
-            if len(self._sn) != self.dimensions():
+            if len(self._sn) != self.dimensions:
                 self._sn = self.sl[slice(0, N)]
                 self._sm = self.si[N-1]
             padded_array[self._sn] = trunc_array[self._sn]
@@ -436,65 +507,37 @@ class R2CBasis(FourierBase):
             k1 = np.fft.fftfreq(Np, 1./Np).astype(int)
             convolve.convolve_real_1D(u, v, uv, k1)
 
-            #u1 = np.hstack((u, np.conj(u[1:][::-1])))
-            #if N % 2 == 0:
-                #u1[N//2:N//2+2] *= 0.5
-            #v1 = np.hstack((v, np.conj(v[1:][::-1])))
-            #if N % 2 == 0:
-                #v1[N//2:N//2+2] *= 0.5
-
-            #for m in range(N):
-                #vc = np.roll(v1, -(m+1))
-                #s = u1*vc[::-1]
-                #ki = k1 + np.roll(k1, -(m+1))[::-1]
-                #z0 = np.argwhere(ki == m)
-                #z1 = np.argwhere(ki == m-N)
-                #uv[m] = np.sum(s[z0])
-                #uv[m-N] = np.sum(s[z1])
-
-            #for m in k1:
-                #for n in k1:
-                    #p = m + n
-                    #if p >= 0:
-                        #if N % 2 == 0:
-                            #if abs(m) == N//2:
-                                #um = u[abs(m)]*0.5
-                            #elif m >= 0:
-                                #um = u[m]
-                            #else:
-                                #um = np.conj(u[abs(m)])
-                            #if abs(n) == N//2:
-                                #vn = v[abs(n)]*0.5
-                            #elif n >= 0:
-                                #vn = v[n]
-                            #else:
-                                #vn = np.conj(v[abs(n)])
-                        #else:
-                            #if m >= 0:
-                                #um = u[m]
-                            #elif m < 0:
-                                #um = np.conj(u[abs(m)])
-                            #if n >= 0:
-                                #vn = v[n]
-                            #elif n < 0:
-                                #vn = np.conj(v[abs(n)])
-                        #uv[p] += um*vn
         return uv
 
 
 class C2CBasis(FourierBase):
     """Fourier basis class for complex to complex transforms
+
+    Parameters
+    ----------
+        N : int
+            Number of quadrature points. Should be even for efficiency, but
+            this is not required.
+        padding_factor : float, optional
+            Factor for padding backward transforms. padding_factor=1.5
+            corresponds to a 3/2-rule for dealiasing.
+        domain : 2-tuple of floats, optional
+            The computational domain.
+        dealias_direct : bool, optional
+            True for dealiasing using 2/3-rule. Must be used with
+            padding_factor = 1.
     """
 
     def __init__(self, N, padding_factor=1., domain=(0., 2.*np.pi),
-                 dealias_direct=False):
-        FourierBase.__init__(self, N, padding_factor, domain, dealias_direct)
+                 dealias_direct=False, **kw):
+        FourierBase.__init__(self, N, padding_factor=padding_factor,
+                             domain=domain, dealias_direct=dealias_direct)
         self.N = N
         #self._xfftn_fwd = pyfftw.builders.fftn
         #self._xfftn_bck = pyfftw.builders.ifftn
         self._xfftn_fwd = fftw.fftn
         self._xfftn_bck = fftw.ifftn
-        self.plan((int(np.floor(padding_factor*N)),), (0,), np.complex, {})
+        self.plan((int(padding_factor*N),), (0,), np.complex, {})
         self._slp = []
 
     def wavenumbers(self, bcast=True, scaled=False, eliminate_highest_freq=False):
@@ -507,11 +550,26 @@ class C2CBasis(FourierBase):
             k = self.broadcast_to_ndims(k)
         return k
 
+    def get_mask_nyquist(self, bcast=True):
+        if self.N % 2 == 0:
+            f = np.ones(self.N, dtype=int)
+            f[self.N//2] = 0
+        else:
+            return None
+        if bcast is True:
+            f = self.broadcast_to_ndims(f)
+        return f
+
     def slice(self):
         return slice(0, self.N)
 
+    def shape(self, forward_output=True):
+        if forward_output:
+            return self.N
+        return int(np.floor(self.padding_factor*self.N))
+
     def _truncation_forward(self, padded_array, trunc_array):
-        if self.padding_factor > 1.0+1e-8:
+        if not id(trunc_array) == id(padded_array):
             trunc_array.fill(0)
             N = trunc_array.shape[self.axis]
             su = self.sl[slice(0, N//2+1)]
@@ -520,10 +578,10 @@ class C2CBasis(FourierBase):
             trunc_array[su] += padded_array[su]
 
     def _padding_backward(self, trunc_array, padded_array):
-        if self.padding_factor > 1.0+1e-8:
+        if not id(trunc_array) == id(padded_array):
             padded_array.fill(0)
             N = trunc_array.shape[self.axis]
-            if len(self._slp) != self.dimensions(): # Store for microoptimization
+            if len(self._slp) != self.dimensions: # Store for microoptimization
                 self._slp = self.sl[slice(0, N//2+1)]
                 self._slm = self.sl[slice(-(N//2), None)]
                 self._slp0 = self.si[N//2]
@@ -580,36 +638,4 @@ class C2CBasis(FourierBase):
             k = np.fft.fftfreq(Np, 1./Np).astype(int)
             convolve.convolve_1D(u, v, uv, k)
 
-            #if N % 2 == 0:
-                #u = np.hstack((u[:N//2], u[N//2], u[N//2:]))
-                #u[N//2:N//2+2] *= 0.5
-                #v = np.hstack((v[:N//2], v[N//2], v[N//2:]))
-                #v[N//2:N//2+2] *= 0.5
-
-            #for m in range(Np):
-                #vc = np.roll(v, -(m+1))
-                #s = u*vc[::-1]
-                #ki = k + np.roll(k, -(m+1))[::-1]
-                #z0 = np.argwhere(ki == m)
-                #z1 = np.argwhere(ki == m-Np)
-                #uv[m] = np.sum(s[z0])
-                #uv[m-Np] = np.sum(s[z1])
-
-            #for m in k:
-                #for n in k:
-                    #p = m + n
-                    #if N % 2 == 0:
-                        #if abs(m) == N//2:
-                            #um = u[m]*0.5
-                        #else:
-                            #um = u[m]
-                        #if abs(n) == N//2:
-                            #vn = v[n]*0.5
-                        #else:
-                            #vn = v[n]
-                    #else:
-                        #um = u[m]
-                        #vn = v[n]
-                    #uv[p] += um*vn
-
         return uv
