@@ -65,7 +65,9 @@ from __future__ import division
 
 #__all__ = ['mat']
 
+import functools
 import numpy as np
+import sympy as sp
 from shenfun.matrixbase import SpectralMatrix
 from shenfun.utilities import inheritdocstrings
 from shenfun.la import TDMA as neumann_TDMA
@@ -77,6 +79,10 @@ LB = bases.Basis
 SD = bases.ShenDirichletBasis
 SB = bases.ShenBiharmonicBasis
 SN = bases.ShenNeumannBasis
+SU = bases.UpperDirichletBasis
+
+x = sp.symbols('x', real=True)
+xp = sp.symbols('x', real=True, positive=True)
 
 #pylint: disable=unused-variable, redefined-builtin, bad-continuation
 
@@ -97,7 +103,7 @@ class BLLmat(SpectralMatrix):
     and :math:`L_k` is the Legendre basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], LB)
         assert isinstance(trial[0], LB)
         N = test[0].N
@@ -105,24 +111,20 @@ class BLLmat(SpectralMatrix):
         d = {0: 2./(2.*k+1)}
         if test[0].quad == 'GL':
             d[0][-1] = 2./(N-1)
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def solve(self, b, u=None, axis=0):
-        N = self.shape[0]
-        assert N == b.shape[axis]
-
+        s = self.trialfunction[0].slice()
         if u is None:
             u = b
         else:
             assert u.shape == b.shape
-
         sl = [np.newaxis]*u.ndim
-        sl[axis] = slice(None)
+        sl[axis] = s
         sl = tuple(sl)
-        d = 1./self[0]
-        d[sl] /= self.scale
-        u[:] = b*d[sl]
-
+        ss = self.trialfunction[0].sl[s]
+        d = (1./self.scale)/self[0]
+        u[ss] = b[ss]*d[sl]
         return u
 
 
@@ -143,7 +145,7 @@ class BDDmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -155,11 +157,11 @@ class BDDmat(SpectralMatrix):
             d[0][-1] = 2./(2*(N-3)+1) + 2./(N-1)
 
         if test[0].is_scaled():
-            d[0] /= 4*k+6
-            d[-2] /= (np.sqrt(4*k[:-2]+6)*np.sqrt(4*k[2:]+6))
+            d[0] /= (4*k+6)
+            d[-2] /= (np.sqrt(4*k[2:]+6)*np.sqrt(4*k[:-2]+6))
 
         d[2] = d[-2]
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self.solve = TDMA(self)
 
 
@@ -180,7 +182,7 @@ class BNNmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Neumann basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SN)
         N = test[0].N
@@ -192,7 +194,7 @@ class BNNmat(SpectralMatrix):
         if test[0].quad == 'GL':
             d[0][-1] = d0[-1] + alpha[-1]**2*2./(N-1)
         d[-2] = d[2]
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         #self.solve = neumann_TDMA(self)
 
 
@@ -213,7 +215,7 @@ class BBBmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Biharmonic basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         from shenfun.la import PDMA
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
@@ -229,7 +231,7 @@ class BBBmat(SpectralMatrix):
              4: gk[:-8]*ek[4:-4]}
         d[-2] = d[2]
         d[-4] = d[4]
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self.solve = PDMA(self)
 
 @inheritdocstrings
@@ -249,16 +251,19 @@ class BDLmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], LB)
         N = test[0].N
         k = np.arange(N, dtype=np.float)
-        d = {2: -2./(2*k[2:] + 1),
-             0: 2./(2.*k[:-2]+1)}
+        sc = np.ones(N)
+        if test[0].is_scaled():
+            sc = 1. / np.sqrt(4*k+6)
+        d = {2: -2./(2*k[2:] + 1)*sc[:-2],
+             0: 2./(2.*k[:-2]+1)*sc[:-2]}
         if test[0].quad == 'GL':
-            d[2][-1] = -2./(N-1)
-        SpectralMatrix.__init__(self, d, test, trial)
+            d[2][-1] = -2./(N-1)*sc[N-3]
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 @inheritdocstrings
 class BLDmat(SpectralMatrix):
@@ -277,16 +282,20 @@ class BLDmat(SpectralMatrix):
     and :math:`\psi_j` is the Shen Legendre Dirichlet basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], LB)
         assert isinstance(trial[0], SD)
         N = test[0].N
         k = np.arange(N, dtype=np.float)
-        d = {-2: -2./(2*k[2:] + 1),
-             0: 2./(2.*k[:-2]+1)}
+        sc = np.ones(N)
+        if trial[0].is_scaled():
+            sc = 1. / np.sqrt(4*k+6)
+
+        d = {-2: -2./(2*k[2:] + 1)*sc[:-2],
+             0: 2./(2.*k[:-2]+1)*sc[:-2]}
         if test[0].quad == 'GL':
-            d[-2][-1] = -2./(N-1)
-        SpectralMatrix.__init__(self, d, test, trial)
+            d[-2][-1] = -2./(N-1)*sc[-3]
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 @inheritdocstrings
 class ADDmat(SpectralMatrix):
@@ -305,7 +314,7 @@ class ADDmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SD)
         N = test[0].N
@@ -314,7 +323,7 @@ class ADDmat(SpectralMatrix):
             d = {0: 4*k+6}
         else:
             d = {0: 1}
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
     def solve(self, b, u=None, axis=0):
         N = self.shape[0] + 2
@@ -331,7 +340,7 @@ class ADDmat(SpectralMatrix):
             # Move axis to first
             if axis > 0:
                 u = np.moveaxis(u, axis, 0)
-                if not u is b:
+                if u is not b:
                     b = np.moveaxis(b, axis, 0)
 
             bs = b[s]
@@ -340,87 +349,21 @@ class ADDmat(SpectralMatrix):
             sl = [np.newaxis]*bs.ndim
             sl[0] = slice(None)
             us[:] = bs*d[tuple(sl)]
-            self.testfunction[0].bc.apply_after(u, True)
+            u /= self.scale
+            self.testfunction[0].bc.set_boundary_dofs(u, True)
 
             if axis > 0:
                 u = np.moveaxis(u, 0, axis)
-                if not u is b:
+                if u is not b:
                     b = np.moveaxis(b, axis, 0)
         else:
             ss = [slice(None)]*b.ndim
             ss[axis] = s
             ss = tuple(ss)
             u[ss] = b[ss]
-            self.testfunction[0].bc.apply_after(u, True)
+            u /= self.scale
+            self.testfunction[0].bc.set_boundary_dofs(u, True)
 
-        u /= self.scale
-        return u
-
-@inheritdocstrings
-class GDDmat(SpectralMatrix):
-    r"""Stiffness matrix for inner product
-
-    .. math::
-
-        G_{kj} = (\psi''_j, \psi_k)_w
-
-    where
-
-    .. math::
-
-        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
-
-    and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
-
-    """
-    def __init__(self, test, trial):
-        assert isinstance(test[0], SD)
-        assert isinstance(trial[0], SD)
-        N = test[0].N
-        k = np.arange(N-2, dtype=np.float)
-        if not test[0].is_scaled():
-            d = {0: -(4*k+6)}
-        else:
-            d = {0: -1}
-        SpectralMatrix.__init__(self, d, test, trial)
-
-    def solve(self, b, u=None, axis=0):
-        N = self.shape[0] + 2
-        assert N == b.shape[0]
-        s = self.trialfunction[0].slice()
-
-        if u is None:
-            u = b
-        else:
-            assert u.shape == b.shape
-
-        if not self.trialfunction[0].is_scaled():
-
-            # Move axis to first
-            if axis > 0:
-                u = np.moveaxis(u, axis, 0)
-                if not u is b:
-                    b = np.moveaxis(b, axis, 0)
-
-            bs = b[s]
-            us = u[s]
-            d = 1./self[0]
-            sl = [np.newaxis]*bs.ndim
-            sl[0] = slice(None)
-            us[:] = bs*d[tuple(sl)]
-            self.testfunction[0].bc.apply_after(u, True)
-
-            if axis > 0:
-                u = np.moveaxis(u, 0, axis)
-                if not u is b:
-                    b = np.moveaxis(b, axis, 0)
-        else:
-            ss = [slice(None)]*b.ndim
-            ss[axis] = s
-            u[ss] = -b[tuple(ss)]
-            self.testfunction[0].bc.apply_after(u, True)
-
-        u /= self.scale
         return u
 
 
@@ -441,7 +384,7 @@ class ANNmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Neumann basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SN)
         N = test[0].N
@@ -449,7 +392,38 @@ class ANNmat(SpectralMatrix):
         alpha = k*(k+1)/(k+2)/(k+3)
         d0 = 2./(2*k+1)
         d = {0: d0*alpha*(k+0.5)*((k+2)*(k+3)-k*(k+1))}
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+    def solve(self, b, u=None, axis=0):
+        N = self.shape[0] + 2
+        assert N == b.shape[axis]
+        s = self.trialfunction[0].slice()
+
+        if u is None:
+            u = b
+        else:
+            assert u.shape == b.shape
+
+
+        # Move axis to first
+        if axis > 0:
+            u = np.moveaxis(u, axis, 0)
+            if u is not b:
+                b = np.moveaxis(b, axis, 0)
+        bs = b[s]
+        us = u[s]
+        d = np.ones(self.shape[0])
+        d[1:] = 1./self[0][1:]
+        sl = [np.newaxis]*bs.ndim
+        sl[0] = slice(None)
+        us[:] = bs*d[tuple(sl)]
+        if axis > 0:
+            u = np.moveaxis(u, 0, axis)
+            if u is not b:
+                b = np.moveaxis(b, axis, 0)
+
+        u /= self.scale
+        return u
 
 
 @inheritdocstrings
@@ -469,7 +443,7 @@ class ABBmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Biharmonic basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, scale=1, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
@@ -478,8 +452,7 @@ class ABBmat(SpectralMatrix):
         d = {0: 2*(2*k+3)*(1+gk),
              2: -2*(2*k[:-2]+3)}
         d[-2] = d[2]
-        SpectralMatrix.__init__(self, d, test, trial)
-
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 @inheritdocstrings
 class GLLmat(SpectralMatrix):
@@ -498,7 +471,7 @@ class GLLmat(SpectralMatrix):
     and :math:`L_k` is the Legendre basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], LB)
         assert isinstance(trial[0], LB)
         N = test[0].N
@@ -507,36 +480,7 @@ class GLLmat(SpectralMatrix):
         for j in range(2, N, 2):
             jj = j if trial[1] else -j
             d[jj] = (k[:-j]+0.5)*((k[:-j]+j)*(k[:-j]+j+1) - k[:-j]*(k[:-j]+1))*2./(2*k[:-j]+1)
-        SpectralMatrix.__init__(self, d, test, trial)
-
-
-@inheritdocstrings
-class PBBmat(SpectralMatrix):
-    r"""Stiffness matrix for inner product
-
-    .. math::
-
-        A_{kj} = (\psi_j, \psi''_k)_w
-
-    where
-
-    .. math::
-
-        j = 0, 1, ..., N-4 \text{ and } k = 0, 1, ..., N-4
-
-    and :math:`\psi_k` is the Shen Legendre Biharmonic basis function.
-
-    """
-    def __init__(self, test, trial):
-        assert isinstance(test[0], SB)
-        assert isinstance(trial[0], SB)
-        N = test[0].N
-        k = np.arange(N-4, dtype=np.float)
-        gk = (2*k+3)/(2*k+7)
-        d = {0: -2*(2*k+3)*(1+gk),
-             2: 2*(2*k[:-2]+3)}
-        d[-2] = d[2]
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
 @inheritdocstrings
@@ -556,13 +500,13 @@ class SBBmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Biharmonic basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
         k = np.arange(N-4, dtype=np.float)
         d = {0: 2*(2*k+3)**2*(2*k+5)}
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 @inheritdocstrings
 class CLLmat(SpectralMatrix):
@@ -581,23 +525,22 @@ class CLLmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], LB)
         assert isinstance(trial[0], LB)
         N = test[0].N
         d = {}
         for i in range(1, N, 2):
             d[i] = 2.0
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def matvec(self, v, c, format='self', axis=0):
-        N = self.shape[0]
         c.fill(0)
         if format == 'self':
             if axis > 0:
                 c = np.moveaxis(c, axis, 0)
                 v = np.moveaxis(v, axis, 0)
-            s = (slice(None),) + (np.newaxis,)*(v.ndim-1) # broadcasting
+            #s = (slice(None),) + (np.newaxis,)*(v.ndim-1) # broadcasting
             ve = v[-2:0:-2].cumsum(axis=0)
             vo = v[-1:0:-2].cumsum(axis=0)
             c[-3::-2] = ve*2.0
@@ -630,14 +573,14 @@ class CLLmatT(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], LB)
         assert isinstance(trial[0], LB)
         N = test[0].N
         d = {}
         for i in range(-1, -N, -2):
             d[i] = 2.0
-        SpectralMatrix.__init__(self, d, test, trial)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 @inheritdocstrings
 class CLDmat(SpectralMatrix):
@@ -656,17 +599,261 @@ class CLDmat(SpectralMatrix):
     and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
 
     """
-    def __init__(self, test, trial):
+    def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], LB)
         assert isinstance(trial[0], SD)
         N = test[0].N
         d = {-1: -2}
-        SpectralMatrix.__init__(self, d, test, trial)
+        if trial[0].is_scaled():
+            k = np.arange(N-2, dtype=np.float)
+            d[-1] = -2. / np.sqrt(4*k+6)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+
+@inheritdocstrings
+class CDLmat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        C_{kj} = (L_j, \psi'_k)_w
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, measure=1):
+        assert isinstance(test[0], SD)
+        assert isinstance(trial[0], LB)
+        N = test[0].N
+        d = {1: -2}
+        if test[0].is_scaled():
+            k = np.arange(N-2, dtype=np.float)
+            d[1] = -2. / np.sqrt(4*k+6)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+
+@inheritdocstrings
+class CDDmat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        C_{kj} = (\psi'_j, \psi_k)_w
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], SD)
+        assert isinstance(trial[0], SD)
+        N = test[0].N
+        d = {-1: -2, 1: 2}
+        if trial[0].is_scaled():
+            k = np.arange(N-2, dtype=np.float)
+            d[-1] = -2. / np.sqrt(4*k[:-1]+6)
+            d[1] = 2. / np.sqrt(4*k[:-1]+6)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+class ADDrp1mat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        A_{kj} = \int_{-1}^{1} \psi_j'(x) \psi_k'(x) (1+x) dx
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], SD)
+        assert isinstance(trial[0], SD)
+        assert test[0].quad == 'LG'
+
+        k = np.arange(test[0].N-2)
+        d = {0: 4*k+6, 1: 2*k[:-1]+4, -1: 2*k[:-1]+4}
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+class ADD2rp1mat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        A_{kj} = \int_{-1}^{1} \psi_j(x) \psi_k''(x) (1+x) dx
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], SD)
+        assert isinstance(trial[0], SD)
+        assert test[0].quad == 'LG'
+
+        k = np.arange(test[0].N-2)
+        d = {0: -(4*k+6), 1: -(2*k[:-1]+6), -1: -(2*k[:-1]+2)}
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+class ADD2Trp1mat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        A_{kj} = \int_{-1}^{1} \psi_j''(x) \psi_k(x) (1+x) dx
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], SD)
+        assert isinstance(trial[0], SD)
+        assert test[0].quad == 'LG'
+
+        k = np.arange(test[0].N-2)
+        d = {0: -(4*k+6), -1: -(2*k[:-1]+6), 1: -(2*k[:-1]+2)}
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+class AUUrp1mat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        A_{kj} = \int_{-1}^{1} \psi_j'(x) \psi_k'(x) (1+x) dx
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Upper Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], SU)
+        assert isinstance(trial[0], SU)
+        assert test[0].quad == 'LG'
+        k = np.arange(test[0].N-1)
+        d = {0: 2*k+2}
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+class BUUrp1mat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        B_{kj} = \int_{-1}^{1} \psi_j(x) \psi_k(x) (1+x) dx
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Upper Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], SU)
+        assert isinstance(trial[0], SU)
+        assert test[0].quad == 'LG'
+        k = np.arange(test[0].N-1)
+        d = {0: 2*k+2}
+        d = {0: 4*(k+1)/(2*k+1)/(2*k+3),
+             1: 4/(2*k[:-1]+1)/(2*k[:-1]+3)/(2*k[:-1]+5),
+             2: -2*(k[:-2]+2)/(2*k[:-2]+3)/(2*k[:-2]+5)}
+        d[-1] = d[1]
+        d[-2] = d[2]
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+
+class BDD1orp1mat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        A_{kj} = \int_{-1}^{1} \psi_j(x) \psi_k(x) 1/(1+x) dx
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], SD)
+        assert isinstance(trial[0], SD)
+        assert test[0].quad == 'LG'
+
+        k = np.arange(test[0].N-2)
+        d = {0: 2*(2*k+3)/(k+1)/(k+2), 1: -2/(k[:-1]+2), -1: -2/(k[:-1]+2)}
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+class BDDrp1mat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        A_{kj} = \int_{-1}^{1} \psi_j(x) \psi_k(x) (1+x) dx
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], SD)
+        assert isinstance(trial[0], SD)
+        assert test[0].quad == 'LG'
+
+        k = np.arange(test[0].N-2)
+        d = {0: 2/(2*k+1)+2/(2*k+5),
+             1: 2/(2*k[:-1]+1)/(2*k[:-1]+5) + 2*(k[:-1]+3)/(2*k[:-1]+5)/(2*k[:-1]+7),
+             2: -2/(2*k[:-2]+5),
+             3: -2*(k[:-3]+3)/(2*k[:-3]+5)/(2*k[:-3]+7)}
+        d[-1] = d[1]
+        d[-2] = d[2]
+        d[-3] = d[3]
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
 
 @inheritdocstrings
 class _Legmatrix(SpectralMatrix):
-    def __init__(self, test, trial):
-        SpectralMatrix.__init__(self, {}, test, trial)
+    def __init__(self, test, trial, measure=1):
+        SpectralMatrix.__init__(self, {}, test, trial, measure=measure)
 
 
 class _LegMatDict(dict):
@@ -686,27 +873,45 @@ class _LegMatDict(dict):
         matrix = dict.__getitem__(self, key)
         return matrix
 
-
 mat = _LegMatDict({
     ((LB, 0), (LB, 0)): BLLmat,
     ((LB, 0), (LB, 1)): CLLmat,
     ((LB, 1), (LB, 0)): CLLmatT,
     ((LB, 0), (SD, 1)): CLDmat,
+    ((SD, 1), (LB, 0)): CDLmat,
+    ((SD, 0), (SD, 1)): CDDmat,
+    ((SD, 1), (SD, 0)): functools.partial(CDDmat, scale=-1.),
     ((SD, 0), (SD, 0)): BDDmat,
     ((SB, 0), (SB, 0)): BBBmat,
     ((SN, 0), (SN, 0)): BNNmat,
     ((SD, 0), (LB, 0)): BDLmat,
     ((LB, 0), (SD, 0)): BLDmat,
     ((SD, 1), (SD, 1)): ADDmat,
-    ((SD, 2), (SD, 0)): GDDmat,
-    ((SD, 0), (SD, 2)): GDDmat,
+    ((SD, 2), (SD, 0)): functools.partial(ADDmat, scale=-1.),
+    ((SD, 0), (SD, 2)): functools.partial(ADDmat, scale=-1.),
     ((SN, 1), (SN, 1)): ANNmat,
+    ((SN, 2), (SN, 0)): functools.partial(ANNmat, scale=-1.),
+    ((SN, 0), (SN, 2)): functools.partial(ANNmat, scale=-1.),
     ((LB, 2), (LB, 0)): GLLmat,
     ((LB, 0), (LB, 2)): GLLmat,
     ((SB, 2), (SB, 2)): SBBmat,
     ((SB, 1), (SB, 1)): ABBmat,
-    ((SB, 0), (SB, 2)): PBBmat,
-    ((SB, 2), (SB, 0)): PBBmat,
+    ((SB, 0), (SB, 2)): functools.partial(ABBmat, scale=-1.),
+    ((SB, 2), (SB, 0)): functools.partial(ABBmat, scale=-1.),
     ((SB, 0), (SB, 4)): SBBmat,
-    ((SB, 4), (SB, 0)): SBBmat
+    ((SB, 4), (SB, 0)): SBBmat,
+    ((SD, 1), (SD, 1), (-1, 1), 1+x): ADDrp1mat,
+    ((SD, 0), (SD, 2), (-1, 1), 1+x): ADD2rp1mat,
+    ((SD, 2), (SD, 0), (-1, 1), 1+x): ADD2Trp1mat,
+    ((SD, 0), (SD, 2), (0, 1), xp): functools.partial(ADD2rp1mat, scale=0.5),
+    ((SD, 2), (SD, 0), (0, 1), xp): functools.partial(ADD2Trp1mat, scale=0.5),
+    ((SD, 1), (SD, 1), (0, 1), xp): functools.partial(ADDrp1mat, scale=0.5),
+    ((SD, 0), (SD, 0), (-1, 1), 1+x): BDDrp1mat,
+    ((SD, 0), (SD, 0), (0, 1), xp): functools.partial(BDDrp1mat, scale=0.5),
+    ((SD, 0), (SD, 0), (-1, 1), 1/(1+x)): BDD1orp1mat,
+    ((SD, 0), (SD, 0), (0, 1), 1/xp): functools.partial(BDD1orp1mat, scale=2),
+    ((SU, 1), (SU, 1), (-1, 1), 1+x): AUUrp1mat,
+    ((SU, 1), (SU, 1), (0, 1), xp): functools.partial(AUUrp1mat, scale=0.5),
+    ((SU, 0), (SU, 0), (-1, 1), 1+x): BUUrp1mat,
+    ((SU, 0), (SU, 0), (0, 1), xp): functools.partial(BUUrp1mat, scale=0.5),
     })
