@@ -1,7 +1,7 @@
 r"""
 This module contains classes for working with the spectral-Galerkin method
 
-There are currently classes for 9 bases and corresponding function spaces
+There are currently classes for 11 bases and corresponding function spaces
 
 All bases have expansions
 
@@ -16,34 +16,34 @@ Chebyshev:
     ChebyshevBasis:
         basis functions: :math:`\phi_k = T_k`
 
-        basis: :math:`span(T_k, k=0,1,..., N)`
+        basis: :math:`span(T_k, k=0,1,..., N-1)`
 
     ShenDirichletBasis:
         basis functions:
             :math:`\phi_k = T_k-T_{k+2}`
 
-            :math:`\phi_{N-1} = 0.5(T_0+T_1)` for Poisson's equation
+            :math:`\phi_{N-2} = 0.5(T_0+T_1)` for Poisson's equation
 
-            :math:`\phi_{N} = 0.5(T_0-T_1)` for Poisson's equation
+            :math:`\phi_{N-1} = 0.5(T_0-T_1)` for Poisson's equation
 
-        basis: :math:`span(\phi_k, k=0,1,...,N)`
+        basis: :math:`span(\phi_k, k=0,1,...,N-1)`
 
-        where :math:`u(1)=a, u(-1)=b`, such that :math:`\hat{u}_{N-1}=a,
-        \hat{u}_{N}=b`.
+        where :math:`u(1)=a, u(-1)=b`, such that :math:`\hat{u}_{N-2}=a,
+        \hat{u}_{N-1}=b`.
 
-        Note that there are only N-1 unknown coefficients, :math:`\hat{u}_k`,
-        since :math:`\hat{u}_{N-1}` and :math:`\hat{u}_{N}` are determined by
+        Note that there are only N-2 unknown coefficients, :math:`\hat{u}_k`,
+        since :math:`\hat{u}_{N-2}` and :math:`\hat{u}_{N-1}` are determined by
         boundary conditions. Inhomogeneous boundary conditions are possible for
-        the Poisson equation, because :math:`\phi_{N}` and :math:`\phi_{N-1}`
+        the Poisson equation, because :math:`\phi_{N-1}` and :math:`\phi_{N-2}`
         are in the kernel of the Poisson operator. For homogeneous boundary
-        conditions :math:`\phi_{N-1}` and :math:`\phi_{N}` are simply ignored.
+        conditions :math:`\phi_{N-2}` and :math:`\phi_{N-1}` are simply ignored.
 
     ShenNeumannBasis:
         basis function:
             :math:`\phi_k = T_k-\left(\frac{k}{k+2}\right)^2T_{k+2}`
 
         basis:
-            :math:`span(\phi_k, k=1,2,...,N-2)`
+            :math:`span(\phi_k, k=1,2,...,N-3)`
 
         Homogeneous Neumann boundary conditions, :math:`u'(\pm 1) = 0`, and
         zero weighted mean: :math:`\int_{-1}^{1}u(x)w(x)dx = 0`.
@@ -53,7 +53,7 @@ Chebyshev:
             :math:`\phi_k = T_k - \frac{2(k+2)}{k+3}T_{k+2} + \frac{k+1}{k+3}T_{k+4}`
 
         basis:
-            :math:`span(\phi_k, k=0,1,...,N-4)`
+            :math:`span(\phi_k, k=0,1,...,N-5)`
 
         Homogeneous Dirichlet and Neumann, :math:`u(\pm 1)=0` and
         :math:`u'(\pm 1)=0`.
@@ -64,35 +64,35 @@ Legendre:
             :math:`\phi_k = L_k`
 
         basis:
-            :math:`span(L_k, k=0,1,...N)`
+            :math:`span(L_k, k=0,1,...N-1)`
 
     ShenDirichletBasis:
         basis function:
             :math:`\phi_k = L_k-L_{k+2}`
 
-            :math:`\phi_{N-1} = 0.5(L_0+L_1)`, for Poisson's equation
+            :math:`\phi_{N-2} = 0.5(L_0+L_1)`, for Poisson's equation
 
-            :math:`\phi_{N} = 0.5(L_0-L_1)`, for Poisson's equation
+            :math:`\phi_{N-1} = 0.5(L_0-L_1)`, for Poisson's equation
 
         basis:
-            :math:`span(\phi_k, k=0,1,...,N)`
+            :math:`span(\phi_k, k=0,1,...,N-1)`
 
         where :math:`u(1)=a, u(-1)=b`, such that
-        :math:`\hat{u}_{N-1}=a, \hat{u}_{N}=b`
+        :math:`\hat{u}_{N-2}=a, \hat{u}_{N-1}=b`
 
-        Note that there are only N-1 unknown coefficients, :math:`\hat{u}_k`,
-        since :math:`\hat{u}_{N-1}` and :math:`\hat{u}_{N}` are determined by
+        Note that there are only N-2 unknown coefficients, :math:`\hat{u}_k`,
+        since :math:`\hat{u}_{N-2}` and :math:`\hat{u}_{N-1}` are determined by
         boundary conditions. Inhomogeneous boundary conditions are possible for
-        the Poisson equation, because :math:`\phi_{N}` and :math:`\phi_{N-1}`
+        the Poisson equation, because :math:`\phi_{N-1}` and :math:`\phi_{N-2}`
         are in the kernel of the Poisson operator. For homogeneous boundary
-        conditions :math:`\phi_{N-1}` and :math:`\phi_{N}` are simply ignored.
+        conditions :math:`\phi_{N-2}` and :math:`\phi_{N-1}` are simply ignored.
 
     ShenNeumannBasis:
         basis function:
             :math:`\phi_k = L_k-\frac{k(k+1)}{(k+2)(k+3)}L_{k+2}`
 
         basis:
-            :math:`span(\phi_k, k=1,2,...,N-2)`
+            :math:`span(\phi_k, k=1,2,...,N-3)`
 
         Homogeneous Neumann boundary conditions, :math:`u'(\pm 1) = 0`, and
         zero mean: :math:`\int_{-1}^{1}u(x)dx = 0`.
@@ -102,41 +102,110 @@ Legendre:
             :math:`\phi_k = L_k - \frac{2(2k+5)}{2k+7}L_{k+2} + \frac{2k+3}{2k+7}L_{k+4}`
 
         basis:
-            :math:`span(\phi_k, k=0,1,...,N-4)`
+            :math:`span(\phi_k, k=0,1,...,N-5)`
 
         Homogeneous Dirichlet and Neumann, :math:`u(\pm 1)=0` and
         :math:`u'(\pm 1)=0`.
 
+Laguerre:
+    LaguerreBasis:
+        basis function:
+            :math:`\phi_k(x) = L_k(x) \cdot \exp(-x)`
+
+        basis:
+            :math:`span(L_k, k=0,1,...N-1)`
+
+        where :math:`L_k` is the Laguerre polynomial of order k.
+
+    ShenDirichletBasis:
+        basis function:
+            :math:`\phi_k = (L_k-L_{k+1})\cdot \exp(-x)`
+
+        basis:
+            :math:`span(\phi_k, k=0,1,...,N-2)`
+
+        Homogeneous Dirichlet for domain [0, inf).
+
+Hermite:
+    Basis:
+        basis function:
+            :math:`\phi_k(x) = H_k(x) \cdot \exp(-x^2/2)/(\pi^{0.25}\sqrt{2^k k!})`
+
+        basis:
+            :math:`span(\phi_k, k=0,1,...N-1)`
+
+        where :math:`H_k` is the Hermite polynomial of order k.
+
+Jacobi:
+    Basis:
+        basis function:
+            :math:`\phi_k = J_k(\alpha, \beta)`
+
+        basis:
+            :math:`span(L_k, k=0,1,...N-1)`
+
+        where :math:`J_k(\alpha, \beta)` is the regular Jacobi polynomial and
+        :math:`\alpha > -1` and :math:`\beta > -1`.
+
+    ShenDirichletBasis:
+        basis function:
+            :math:`\phi_k = j_k(-1, -1)`
+
+        basis:
+            :math:`span(\phi_k, k=0,1,...,N-3)`
+
+        where :math:`j_k` is the generalized Jacobi polynomial
+
+    ShenBiharmonicBasis:
+        basis function:
+            :math:`\phi_k = j_k(-2, -2)`
+
+        basis:
+            :math:`span(\phi_k, k=0,1,...,N-5)`
+
+        Homogeneous Dirichlet and Neumann, :math:`u(\pm 1)=0` and
+        :math:`u'(\pm 1)=0`.
+
+    ShenOrder6Basis:
+        basis function:
+            :math:`\phi_k = j_k(-3, -3)`
+
+        basis:
+            :math:`span(\phi_k, k=0,1,...,N-7)`
+
+        Homogeneous :math:`u(\pm 1)=u'(\pm 1)=u''(\pm 1)=0`.
+
 Fourier:
     R2CBasis and C2CBasis:
         basis function:
-            :math:`\phi_k = c_k exp(ikx)`
+            :math:`\phi_k = exp(ikx)`
 
         basis:
-            :math:`span(\phi_k, k=-N/2, -N/2+1, ..., N/2)`
+            :math:`span(\phi_k, k=-N/2, -N/2+1, ..., N/2-1)`
 
-        If the basis is used as an interpolator (if the solution is to be
-        obtained in physical space) and N is even, then :math:`c_{-N/2}`
-        and :math:`c_{N/2} = 0.5` and :math:`c_k = 1` for
-        :math:`k=-N/2+1, ..., N/2-1`. :math:`i` is the imaginary unit.
-
-        If N is odd, then :math:`c_k = 1` for :math:`k=-N/2, ..., N/2`.
-
-        Note that by default a truncation form is used and in that case
-        :math:`c_k=1` for all :math:`k`.
+        Note that if N is even, then the Nyquist frequency (-N/2) requires
+        special attention for the R2CBasis. We should then have been using
+        a Fourier interpolator that symmetrizes by adding and extra
+        :math:`\phi_{N/2}` and by multiplying :math:`\phi_{N/2}` and
+        :math:`\phi_{-N/2}` with 0.5. This effectively sets the
+        Nyquist frequency (-N/2) to zero for odd derivatives. This is
+        not done automatically by shenfun. Instead we recommend using
+        the :func:`.SpectralBase.mask_nyquist` function that effectively sets the
+        Nyquist frequency to zero (if N is even).
 
     R2CBasis and C2CBasis are the same, but R2CBasis is used on real physical
     data and it takes advantage of Hermitian symmetry,
     :math:`\hat{u}_{-k} = conj(\hat{u}_k)`, for :math:`k = 1, ..., N/2`
 
-
-Each class has methods for moving fast between spectral and physical space, and
+Each class has methods for moving (fast) between spectral and physical space, and
 for computing the (weighted) scalar product.
 
 """
 #pylint: disable=unused-argument, not-callable, no-self-use, protected-access, too-many-public-methods, missing-docstring
 
 import importlib
+import warnings
+import sympy as sp
 import numpy as np
 from mpi4py_fft import fftw
 from .utilities import CachedArrayDict
@@ -154,34 +223,45 @@ class SpectralBase(object):
 
             - GL - Chebyshev-Gauss-Lobatto or Legendre-Gauss-Lobatto
             - GC - Chebyshev-Gauss
-            - LG - Legendre-Gauss
+            - LG - Legendre-Gauss or Laguerre-Gauss
+            - HG - Hermite-Gauss
 
         padding_factor : float, optional
             For padding backward transform (for dealiasing)
         domain : 2-tuple of floats, optional
             The computational domain
+        dealias_direct : bool, optional
+            If True, then set all upper 2/3 wavenumbers to zero before backward
+            transform. Cannot be used if padding_factor is different than 1.
 
     """
     # pylint: disable=method-hidden, too-many-instance-attributes
 
-    def __init__(self, N, quad, padding_factor=1, domain=(-1., 1.)):
+    def __init__(self, N, quad='', padding_factor=1, domain=(-1., 1.), dealias_direct=False):
         self.N = N
         self.domain = domain
         self.quad = quad
         self.axis = 0
         self.xfftn_fwd = None
         self.xfftn_bck = None
+        self.bc = None
         self.padding_factor = np.floor(N*padding_factor)/N
+        self.dealias_direct = dealias_direct
         self._mass = None         # Mass matrix (if needed)
         self._xfftn_fwd = None    # external forward transform function
         self._xfftn_bck = None    # external backward transform function
         self._M = 1.0             # Normalization factor
+        self._dx = 1              # Integral measure (in addition to weight)
         self.si = islicedict()
         self.sl = slicedict()
         self._tensorproductspace = None     # link if belonging to TensorProductSpace
 
-    def points_and_weights(self, N=None, map_true_domain=False):
-        """Return points and weights of quadrature
+    def points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
+        r"""Return points and weights of quadrature for weighted integral
+
+        .. math::
+
+            \int_{\Omega} f(x) w(x) dx \approx \sum_{i} f(x_i) w_i
 
         Parameters
         ----------
@@ -189,10 +269,38 @@ class SpectralBase(object):
                 Number of quadrature points
             map_true_domain : bool, optional
                 Whether or not to map points to true domain
+            weighted : bool, optional
+                Whether to use quadrature weights for a weighted inner product
+                (default), or a regular, non-weighted inner product.
         """
         raise NotImplementedError
 
-    def mesh(self, bcast=True, map_true_domain=True):
+    def mpmath_points_and_weights(self, N=None, map_true_domain=False, weighted=True, **kw):
+        r"""Return points and weights of quadrature using extended precision
+        mpmath if available
+
+        .. math::
+
+            \int_{\Omega} f(x) w(x) dx \approx \sum_{i} f(x_i) w_i
+
+        Parameters
+        ----------
+            N : int, optional
+                Number of quadrature points
+            map_true_domain : bool, optional
+                Whether or not to map points to true domain
+            weighted : bool, optional
+                Whether to use quadrature weights for a weighted inner product
+                (default), or a regular, non-weighted inner product.
+
+        Note
+        ----
+        If not implemented, or if mpmath/quadpy are not available, then simply
+        returns the regular numpy :func:`points_and_weights`.
+        """
+        return self.points_and_weights(N=N, map_true_domain=map_true_domain, weighted=weighted, **kw)
+
+    def mesh(self, bcast=True, map_true_domain=True, uniform=False):
         """Return the computational mesh
 
         Parameters
@@ -202,8 +310,17 @@ class SpectralBase(object):
                 if basis belongs to a :class:`.TensorProductSpace`
             map_true_domain : bool, optional
                 Whether or not to map points to true domain
+            uniform : bool, optional
+                Use uniform mesh instead of quadrature if True
         """
-        X = self.points_and_weights(map_true_domain=map_true_domain)[0]
+        N = self.shape(False)
+        if self.family() == 'fourier':
+            uniform = False
+        if uniform is False:
+            X = self.points_and_weights(N=N, map_true_domain=map_true_domain)[0]
+        else:
+            d = self.domain
+            X = np.linspace(d[0], d[1], N)
         if bcast is True:
             X = self.broadcast_to_ndims(X)
         return X
@@ -251,7 +368,7 @@ class SpectralBase(object):
         >>> print(y.shape)
         (4, 1)
         """
-        s = [np.newaxis]*self.dimensions()
+        s = [np.newaxis]*self.dimensions
         s[self.axis] = slice(None)
         return x[tuple(s)]
 
@@ -277,6 +394,8 @@ class SpectralBase(object):
         if input_array is not None:
             self.scalar_product.input_array[...] = input_array
 
+        self.scalar_product._input_array = self.get_measured_array(self.scalar_product._input_array)
+
         self.evaluate_scalar_product(self.scalar_product.input_array,
                                      self.scalar_product.output_array,
                                      fast_transform=fast_transform)
@@ -321,9 +440,9 @@ class SpectralBase(object):
         Parameters
         ----------
             input_array : array, optional
-                Function values on quadrature mesh
-            output_array : array, optional
                 Expansion coefficients
+            output_array : array, optional
+                Function values on quadrature mesh
             fast_transform : bool, optional
                 If True use fast transforms (if implemented), if
                 False use Vandermonde type
@@ -349,6 +468,40 @@ class SpectralBase(object):
             return output_array
         return self.backward.output_array
 
+    def backward_uniform(self, input_array=None, output_array=None):
+        """Evaluate function on uniform mesh
+
+        Parameters
+        ----------
+            input_array : array, optional
+                Expansion coefficients
+            output_array : array, optional
+                Function values on quadrature mesh
+
+        Note
+        ----
+        If input_array/output_array are not given, then use predefined arrays
+        as planned with self.plan
+
+        """
+        if self.family() == 'fourier': # Fourier is already using uniform mesh. Use fast transform.
+            return self.backward(input_array, output_array)
+
+        if input_array is not None:
+            self.backward.input_array[...] = input_array
+
+        self._padding_backward(self.backward.input_array,
+                               self.backward.tmp_array)
+
+        self.vandermonde_evaluate_expansion_all(self.backward.tmp_array,
+                                                self.backward.output_array,
+                                                x=self.mesh(bcast=False, uniform=True))
+
+        if output_array is not None:
+            output_array[...] = self.backward.output_array
+            return output_array
+        return self.backward.output_array
+
     def vandermonde(self, x):
         r"""Return Vandermonde matrix based on the primary basis of the family.
 
@@ -399,13 +552,15 @@ class SpectralBase(object):
         """
         raise NotImplementedError
 
-    def evaluate_basis_all(self, x=None):
+    def evaluate_basis_all(self, x=None, argument=0):
         """Evaluate basis at ``x`` or all quadrature points
 
         Parameters
         ----------
             x : float or array of floats, optional
                 If not provided use quadrature points of self
+            argument : int
+                Zero for test and 1 for trialfunction
 
         Returns
         -------
@@ -436,9 +591,18 @@ class SpectralBase(object):
                 output_array
 
         """
-        raise NotImplementedError
+        warnings.warn('Using slow sympy evaluate_basis_derivative')
+        if x is None:
+            x = self.mesh(False, False)
+        if output_array is None:
+            output_array = np.zeros(x.shape)
+        x = np.atleast_1d(x)
+        X = sp.symbols('x')
+        basis = self.sympy_basis(i=i, x=X).diff(X, k)
+        output_array[:] = sp.lambdify(X, basis, 'numpy')(x)
+        return output_array
 
-    def evaluate_basis_derivative_all(self, x=None, k=0):
+    def evaluate_basis_derivative_all(self, x=None, k=0, argument=0):
         """Return k'th derivative of basis evaluated at ``x`` or all quadrature
         points as a Vandermonde matrix.
 
@@ -448,6 +612,8 @@ class SpectralBase(object):
                 If not provided use quadrature points of self
             k : int, optional
                 k'th derivative
+            argument : int
+                Zero for test and 1 for trialfunction
 
         Returns
         -------
@@ -484,25 +650,21 @@ class SpectralBase(object):
                 Expansion coefficients
 
         """
-        assert abs(self.padding_factor-1) < 1e-8
-        assert self.N == input_array.shape[self.axis]
-
-        _, weights = self.points_and_weights()
-        P = self.evaluate_basis_all()
-
+        weights = self.points_and_weights(int(self.N*self.padding_factor))[1]
+        P = self.evaluate_basis_all(argument=0)
         if input_array.ndim == 1:
-            output_array[:] = np.dot(input_array*weights, np.conj(P))
+            output_array[slice(0, self.N)] = np.dot(input_array*weights, np.conj(P))
 
         else: # broadcasting
             bc_shape = [np.newaxis,]*input_array.ndim
             bc_shape[self.axis] = slice(None)
             fc = np.moveaxis(input_array*weights[tuple(bc_shape)], self.axis, -1)
-            output_array[:] = np.moveaxis(np.dot(fc, np.conj(P)), -1, self.axis)
+            output_array[self.sl[slice(0, self.N)]] = np.moveaxis(np.dot(fc, np.conj(P)), -1, self.axis)
             #output_array[:] = np.moveaxis(np.tensordot(input_array*weights[bc_shape], np.conj(P), (self.axis, 0)), -1, self.axis)
 
-        assert output_array is self.forward.output_array
+        assert output_array is self.scalar_product.output_array
 
-    def vandermonde_evaluate_expansion_all(self, input_array, output_array):
+    def vandermonde_evaluate_expansion_all(self, input_array, output_array, x=None):
         """Naive implementation of evaluate_expansion_all
 
         Parameters
@@ -511,21 +673,17 @@ class SpectralBase(object):
                 Expansion coefficients
             output_array : array
                 Function values on quadrature mesh
-
-        Note
-        ----
-        Implemented only for non-padded transforms
+            x : mesh or None, optional
 
         """
-        assert abs(self.padding_factor-1) < 1e-8
-        assert self.N == output_array.shape[self.axis]
-        P = self.evaluate_basis_all()
-
+        P = self.evaluate_basis_all(x=x, argument=1)
         if output_array.ndim == 1:
             output_array = np.dot(P, input_array, out=output_array)
         else:
             fc = np.moveaxis(input_array, self.axis, -2)
-            array = np.dot(P, fc)
+            shape = [slice(None)]*input_array.ndim
+            shape[-2] = slice(0, self.N)
+            array = np.dot(P, fc[tuple(shape)])
             output_array[:] = np.moveaxis(array, 0, self.axis)
 
     def vandermonde_evaluate_expansion(self, points, input_array, output_array):
@@ -542,8 +700,7 @@ class SpectralBase(object):
                 Function values on points
 
         """
-        assert abs(self.padding_factor-1) < 1e-8
-        P = self.evaluate_basis_all(x=points)
+        P = self.evaluate_basis_all(x=points, argument=1)
 
         if output_array.ndim == 1:
             output_array = np.dot(P, input_array, out=output_array)
@@ -563,8 +720,12 @@ class SpectralBase(object):
                 Expansion coefficients. Overwritten by applying the inverse
                 mass matrix, and returned.
 
+        Returns
+        -------
+            array
+
         """
-        assert self.N == array.shape[self.axis]
+        #assert self.N == array.shape[self.axis]
         if self._mass is None:
             B = self.get_mass_matrix()
             self._mass = B((self, 0), (self, 0))
@@ -572,6 +733,24 @@ class SpectralBase(object):
         array = self._mass.solve(array, axis=self.axis)
         return array
 
+    def to_ortho(self, input_array, output_array=None):
+        """Project to orthogonal basis
+
+        Parameters
+        ----------
+            input_array : array
+                Expansion coefficients of input basis
+            output_array : array, optional
+                Expansion coefficients in orthogonal basis
+
+        Returns
+        -------
+            array
+                output_array
+
+        """
+        raise NotImplementedError
+
     def plan(self, shape, axis, dtype, options):
         """Plan transform
 
@@ -600,7 +779,7 @@ class SpectralBase(object):
         plan_fwd = self._xfftn_fwd
         plan_bck = self._xfftn_bck
 
-        if 'builders' in self._xfftn_fwd.__module__:
+        if 'builders' in self._xfftn_fwd.__module__: #pragma: no cover
 
             opts = dict(
                 avoid_copy=True,
@@ -609,6 +788,7 @@ class SpectralBase(object):
                 auto_contiguous=True,
                 planner_effort='FFTW_MEASURE',
                 threads=1,
+                backward=False,
             )
             opts.update(options)
 
@@ -654,17 +834,19 @@ class SpectralBase(object):
             trunc_array = self._get_truncarray(shape, V.dtype)
             self.forward = Transform(self.forward, xfftn_fwd, U, V, trunc_array)
             self.backward = Transform(self.backward, xfftn_bck, trunc_array, V, U)
+            self.backward_uniform = Transform(self.backward_uniform, xfftn_bck, trunc_array, V, U)
         else:
             self.forward = Transform(self.forward, xfftn_fwd, U, V, V)
             self.backward = Transform(self.backward, xfftn_bck, V, V, U)
+            self.backward_uniform = Transform(self.backward_uniform, xfftn_bck, V, V, U)
 
         # scalar_product is not padded, just the forward/backward
         self.scalar_product = Transform(self.scalar_product, xfftn_fwd, U, V, V)
-        self.si = islicedict(axis=self.axis, dimensions=self.dimensions())
-        self.sl = slicedict(axis=self.axis, dimensions=self.dimensions())
+        self.si = islicedict(axis=self.axis, dimensions=self.dimensions)
+        self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
 
     def _get_truncarray(self, shape, dtype):
-        shape = list(shape)
+        shape = list(shape) if np.ndim(shape) else [shape]
         shape[self.axis] = int(np.round(shape[self.axis] / self.padding_factor))
         return fftw.aligned(shape, dtype=dtype)
 
@@ -729,6 +911,24 @@ class SpectralBase(object):
             x = a + (x-c)/self.domain_factor()
         return x
 
+    def sympy_basis(self, i=0, x=None):
+        """Return basis function `i` as sympy function
+
+        Parameters
+        ----------
+        i : int, optional
+            The degree of freedom of the basis function
+        x : sympy Symbol, optional
+        """
+        raise NotImplementedError
+
+    def sympy_basis_all(self, x=None):
+        """Return all basis functions as sympy functions"""
+        return np.array([self.sympy_basis(i, x=x) for i in range(self.slice().start, self.slice().stop)])
+
+    def sympy_weight(self, x=None):
+        return 1
+
     def reference_domain(self):
         """Return reference domain of basis"""
         raise NotImplementedError
@@ -737,33 +937,29 @@ class SpectralBase(object):
         """Return index set of current basis"""
         return slice(0, self.N)
 
+    def dim(self):
+        """Return the dimension of ``self`` (the number of degrees of freedom)"""
+        s = self.slice()
+        return s.stop - s.start
+
+    def dims(self):
+        """Return tuple (length one since a basis only has one dim) containing
+        self.dim()"""
+        return (self.dim(),)
+
     def shape(self, forward_output=True):
-        """Return the shape of current basis
-
-        Parameters
-        ----------
-            forward_output : bool, optional
-                If True then return shape of spectral space (the result of a
-                forward transform). If False then return shape of physical space
-                (the input to a forward transform).
-        """
-        if forward_output:
-            s = self.slice()
-            return s.stop - s.start
-        return self.N
-
-    def allocated_shape(self, forward_output=True):
-        """Return allocated shape for basis
+        """Return the allocated shape of arrays used for ``self``
 
         Parameters
         ----------
         forward_output : bool, optional
-            If True then return allocated shape for spectral space (the result
-            of a forward transform). If False then return allocated shape of
-            physical space (the input to a forward transform).
-
+            If True then return allocated shape of spectral space (the result of a
+            forward transform). If False then return allocated shape of physical space
+            (the input to a forward transform).
         """
-        return self.N
+        if forward_output:
+            return self.N
+        return int(np.floor(self.padding_factor*self.N))
 
     def domain_factor(self):
         """Return scaling factor for domain"""
@@ -775,12 +971,13 @@ class SpectralBase(object):
             return 1
         return R/L
 
+    @property
     def dimensions(self):
         """Return the dimensions (the number of bases) of the
         :class:`.TensorProductSpace` class this basis is planned for.
         """
         if self.tensorproductspace:
-            return self.tensorproductspace.dimensions()
+            return self.tensorproductspace.dimensions
         return self.forward.input_array.ndim
 
     @property
@@ -799,13 +996,21 @@ class SpectralBase(object):
     def tensorproductspace(self, T):
         self._tensorproductspace = T
 
-    def __hash__(self):
-        return hash(repr(self.__class__))
-
     def __eq__(self, other):
         return (self.__class__.__name__ == other.__class__.__name__ and
-                self.quad == other.quad and self.N == other.N)
+                self.quad == other.quad and
+                self.N == other.N and
+                self.axis == other.axis)
 
+    @property
+    def is_orthogonal(self):
+        return False
+
+    @property
+    def has_nonhomogeneous_bcs(self):
+        return False
+
+    @property
     def rank(self):
         """Return tensor rank of basis"""
         return 0
@@ -829,12 +1034,22 @@ class SpectralBase(object):
 
     def get_mass_matrix(self):
         mat = self._get_mat()
-        return mat[((self.__class__, 0), (self.__class__, 0))]
+        dx = self._dx
+        if not dx == 1:
+            x0 = dx.free_symbols
+            assert len(x0) == 1
+            x0 = x0.pop()
+            x = sp.symbols('x', real=x0.is_real, positive=x0.is_positive)
+            dx = dx.subs(x0, x)
+        return mat[((self.__class__, 0), (self.__class__, 0), dx)]
 
     def _get_mat(self):
         mod = importlib.import_module('shenfun.'+self.family())
         return mod.matrices.mat
 
+    def compatible_base(self, space):
+        return hash(self) == hash(space)
+
     def __len__(self):
         return 1
 
@@ -842,14 +1057,115 @@ class SpectralBase(object):
         assert i == 0
         return self
 
+    def __hash__(self):
+        return hash((self.N, self.quad, self.family()))
+
+    def get_measured_weights(self, N=None, measure=None):
+        """Return weights times `measure`
+
+        Parameters
+        ----------
+        N : integer, optional
+            The number of quadrature points
+        measure : None or `sympy.Expr`
+        """
+        if N is None:
+            N = self.N
+        dx = self._dx if measure is None else measure
+        xm, wj = self.mpmath_points_and_weights(N, map_true_domain=True)
+        if dx == 1:
+            return wj
+
+        s = dx.free_symbols
+        assert len(s) == 1
+        s = s.pop()
+        xj = sp.lambdify(s, dx)(xm)
+        wj *= xj
+        return wj
+
+    def get_measured_array(self, input_array, measure=None):
+        """Return `input_array` times `measure`
+
+        Parameters
+        ----------
+        input_array : Array
+        measure : None or `sympy.Expr`
+        """
+        dx = self._dx if measure is None else measure
+        xm = self.mpmath_points_and_weights(self.N, map_true_domain=True)[0]
+        if dx == 1:
+            return input_array
+
+        s = dx.free_symbols
+        assert len(s) == 1
+        s = s.pop()
+        xj = sp.lambdify(s, dx)(xm)
+        xj = self.broadcast_to_ndims(xj)
+        input_array *= xj
+        return input_array
+
+    def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
+        """Return space (otherwise as self) to be used for dealiasing
+
+        Parameters
+        ----------
+        padding_factor : float, optional
+            Create output array of shape padding_factor time non-padded shape
+        dealias_direct : bool, optional
+            If True, set upper 2/3 of wavenumbers to zero before backward transform.
+            Cannot be used together with padding_factor different than 1.
+
+        Returns
+        -------
+        SpectralBase
+            The space to be used for dealiasing
+        """
+        return self.__class__(self.N,
+                              quad=self.quad,
+                              domain=self.domain,
+                              padding_factor=padding_factor,
+                              dealias_direct=dealias_direct)
+
+    def get_refined(self, N):
+        """Return space (otherwise as self) with N quadrature points
+
+        Parameters
+        ----------
+        N : int
+            The number of quadrature points for returned space
+
+        Returns
+        -------
+        SpectralBase
+            A new space with new number of quadrature points, otherwise as self.
+        """
+        return self.__class__(N,
+                              quad=self.quad,
+                              domain=self.domain,
+                              padding_factor=self.padding_factor,
+                              dealias_direct=self.dealias_direct)
+
     def _truncation_forward(self, padded_array, trunc_array):
-        pass
+        if not id(trunc_array) == id(padded_array):
+            trunc_array.fill(0)
+            N = trunc_array.shape[self.axis]
+            s = self.sl[slice(0, N)]
+            trunc_array[:] = padded_array[s]
 
     def _padding_backward(self, trunc_array, padded_array):
-        pass
+        if not id(trunc_array) == id(padded_array):
+            padded_array.fill(0)
+            N = trunc_array.shape[self.axis]
+            _sn = self.sl[slice(0, N)]
+            padded_array[_sn] = trunc_array[_sn]
+
+        elif self.dealias_direct:
+            su = self.sl[slice(2*self.N//3, None)]
+            padded_array[su] = 0
+
 
 class MixedBasis(object):
-    """Class for composite bases
+    """Class for composite bases in 1D
 
     Parameters
     ----------
@@ -863,6 +1179,20 @@ class MixedBasis(object):
         self.backward = VectorBasisTransform([basis.backward for basis in bases])
         self.scalar_product = VectorBasisTransform([basis.scalar_product for basis in bases])
 
+    def dims(self):
+        """Return dimensions (degrees of freedom) for MixedBasis"""
+        s = []
+        for space in self.flatten():
+            s.append(space.dim())
+        return s
+
+    def dim(self):
+        """Return dimension of ``self`` (degrees of freedom)"""
+        s = 0
+        for space in self.flatten():
+            s += space.dim()
+        return s
+
     def shape(self, forward_output=False):
         """Return shape of arrays for MixedBasis
 
@@ -873,8 +1203,14 @@ class MixedBasis(object):
             forward transform. If False then return shape of physical
             space, i.e., the input to a forward transform.
         """
-        s = self.bases[0].shape(forward_output)
-        return (self.num_components(),) + s
+        if forward_output:
+            s = []
+            for space in self.flatten():
+                s.append(space.shape(forward_output))
+        else:
+            s = self.flatten()[0].shape(forward_output)
+            s = (self.num_components(),) + s
+        return s
 
     def num_components(self):
         """Return number of bases in mixed basis"""
@@ -900,14 +1236,16 @@ class MixedBasis(object):
         return getattr(obj[0], name)
 
     def __len__(self):
-        return self.bases[0].dimensions()
+        return self.bases[0].dimensions
 
+    @property
     def rank(self):
         return 1
 
+    @property
     def dimensions(self):
         """Return dimension of scalar space"""
-        return self.bases[0].dimensions()
+        return self.bases[0].dimensions
 
 class VectorBasisTransform(object):
 
@@ -1001,8 +1339,8 @@ class slicedict(dict):
         return dict.__getitem__(self, self.__keytransform__(key))
 
 
-def inner_product(test, trial):
-    """Return 1D inner product of bilinear form
+def inner_product(test, trial, measure=1):
+    """Return 1D weighted inner product of bilinear form
 
     Parameters
     ----------
@@ -1012,11 +1350,15 @@ def inner_product(test, trial):
             - :mod:`.chebyshev.bases`
             - :mod:`.legendre.bases`
             - :mod:`.fourier.bases`
+            - :mod:`.laguerre.bases`
+            - :mod:`.hermite.bases`
+            - :mod:`.jacobi.bases`
 
             The integer determines the number of times the basis is
             differentiated. The test represents the matrix row
         trial : 2-tuple of (Basis, integer)
             Like test
+        measure: function of coordinate, optional
 
     Note
     ----
@@ -1039,20 +1381,36 @@ def inner_product(test, trial):
     """
     assert trial[0].__module__ == test[0].__module__
     key = ((test[0].__class__, test[1]), (trial[0].__class__, trial[1]))
+    sc = 1
+    if measure != 1:
+        x0 = measure.free_symbols
+        assert len(x0) == 1
+        x0 = x0.pop()
+        x = sp.symbols('x', real=x0.is_real, positive=x0.is_positive)
+        measure = measure.subs(x0, x)
+        if measure.subs(x, 1) < 0:
+            sc = -1
+            measure *= sc
+        key = key + (test[0].domain, measure)
+
     mat = test[0]._get_mat()
-    return mat[key](test, trial)
+    A = mat[key](test, trial, measure=measure)
+    A.scale *= sc
+    if not test[0].domain_factor() == 1:
+        A.scale *= test[0].domain_factor()**(test[1]+trial[1])
+    return A
 
 class FuncWrap(object):
 
     # pylint: disable=too-few-public-methods, missing-docstring
 
-    __slots__ = ('_func', '__doc__', '_input_array', '_output_array')
+    __slots__ = ('_func', '_input_array', '_output_array')
 
     def __init__(self, func, input_array, output_array):
         object.__setattr__(self, '_func', func)
         object.__setattr__(self, '_input_array', input_array)
         object.__setattr__(self, '_output_array', output_array)
-        object.__setattr__(self, '__doc__', func.__doc__)
+        #object.__setattr__(self, '__doc__', func.__doc__)
 
     @property
     def input_array(self):
@@ -1073,7 +1431,7 @@ class Transform(FuncWrap):
 
     # pylint: disable=too-few-public-methods
 
-    __slots__ = ('_xfftn', '__doc__', '_input_array', '_output_array',
+    __slots__ = ('_xfftn', '_input_array', '_output_array',
                  '_tmp_array')
 
     def __init__(self, func, xfftn, input_array, tmp_array, output_array):
